@@ -17,30 +17,32 @@ from .. import build
 from .. import mesonlib
 import uuid, os, sys
 
-from ..coredata import MesonException
+from ..mesonlib import MesonException
 
 class XCodeBackend(backends.Backend):
     def __init__(self, build):
         super().__init__(build)
+        self.name = 'xcode'
         self.project_uid = self.environment.coredata.guid.replace('-', '')[:24]
         self.project_conflist = self.gen_id()
         self.indent = '       '
         self.indent_level = 0
-        self.xcodetypemap = {'c' : 'sourcecode.c.c',
-                             'a' : 'archive.ar',
+        self.xcodetypemap = {'c': 'sourcecode.c.c',
+                             'a': 'archive.ar',
                              'cc': 'sourcecode.cpp.cpp',
-                             'cxx' : 'sourcecode.cpp.cpp',
-                             'cpp' : 'sourcecode.cpp.cpp',
-                             'c++' : 'sourcecode.cpp.cpp',
-                             'm' : 'sourcecode.c.objc',
-                             'mm' : 'sourcecode.cpp.objcpp',
-                             'h' : 'sourcecode.c.h',
-                             'hpp' : 'sourcecode.cpp.h',
-                             'hxx' : 'sourcecode.cpp.h',
-                             'hh' : 'sourcecode.cpp.hh',
-                             'inc' : 'sourcecode.c.h',
-                             'dylib' : 'compiled.mach-o.dylib',
-                             'o' : 'compiled.mach-o.objfile',}
+                             'cxx': 'sourcecode.cpp.cpp',
+                             'cpp': 'sourcecode.cpp.cpp',
+                             'c++': 'sourcecode.cpp.cpp',
+                             'm': 'sourcecode.c.objc',
+                             'mm': 'sourcecode.cpp.objcpp',
+                             'h': 'sourcecode.c.h',
+                             'hpp': 'sourcecode.cpp.h',
+                             'hxx': 'sourcecode.cpp.h',
+                             'hh': 'sourcecode.cpp.hh',
+                             'inc': 'sourcecode.c.h',
+                             'dylib': 'compiled.mach-o.dylib',
+                             'o': 'compiled.mach-o.objfile',
+                             }
         self.maingroup_id = self.gen_id()
         self.all_id = self.gen_id()
         self.all_buildconf_id = self.gen_id()
@@ -58,13 +60,13 @@ class XCodeBackend(backends.Backend):
         return dirname
 
     def write_line(self, text):
-        self.ofile.write(self.indent*self.indent_level + text)
+        self.ofile.write(self.indent * self.indent_level + text)
         if not text.endswith('\n'):
             self.ofile.write('\n')
 
     def generate(self, interp):
         self.interpreter = interp
-        self.serialise_tests()
+        test_data = self.serialise_tests()[0]
         self.generate_filemap()
         self.generate_buildmap()
         self.generate_buildstylemap()
@@ -82,22 +84,22 @@ class XCodeBackend(backends.Backend):
         self.proj_dir = os.path.join(self.environment.get_build_dir(), self.build.project_name + '.xcodeproj')
         os.makedirs(self.proj_dir, exist_ok=True)
         self.proj_file = os.path.join(self.proj_dir, 'project.pbxproj')
-        self.ofile = open(self.proj_file, 'w')
-        self.generate_prefix()
-        self.generate_pbx_aggregate_target()
-        self.generate_pbx_build_file()
-        self.generate_pbx_build_style()
-        self.generate_pbx_container_item_proxy()
-        self.generate_pbx_file_reference()
-        self.generate_pbx_group()
-        self.generate_pbx_native_target()
-        self.generate_pbx_project()
-        self.generate_pbx_shell_build_phase()
-        self.generate_pbx_sources_build_phase()
-        self.generate_pbx_target_dependency()
-        self.generate_xc_build_configuration()
-        self.generate_xc_configurationList()
-        self.generate_suffix()
+        with open(self.proj_file, 'w') as self.ofile:
+            self.generate_prefix()
+            self.generate_pbx_aggregate_target()
+            self.generate_pbx_build_file()
+            self.generate_pbx_build_style()
+            self.generate_pbx_container_item_proxy()
+            self.generate_pbx_file_reference()
+            self.generate_pbx_group()
+            self.generate_pbx_native_target()
+            self.generate_pbx_project()
+            self.generate_pbx_shell_build_phase(test_data)
+            self.generate_pbx_sources_build_phase()
+            self.generate_pbx_target_dependency()
+            self.generate_xc_build_configuration()
+            self.generate_xc_configurationList()
+            self.generate_suffix()
 
     def get_xcodetype(self, fname):
         return self.xcodetypemap[fname.split('.')[-1]]
@@ -128,7 +130,7 @@ class XCodeBackend(backends.Backend):
                     self.buildmap[o] = self.gen_id()
 
     def generate_buildstylemap(self):
-        self.buildstylemap = {'debug' : self.gen_id()}
+        self.buildstylemap = {'debug': self.gen_id()}
 
     def generate_build_phase_map(self):
         self.buildphasemap = {}
@@ -138,17 +140,17 @@ class XCodeBackend(backends.Backend):
     def generate_build_configuration_map(self):
         self.buildconfmap = {}
         for t in self.build.targets:
-            bconfs = {'debug' : self.gen_id()}
+            bconfs = {'debug': self.gen_id()}
             self.buildconfmap[t] = bconfs
 
     def generate_project_configurations_map(self):
-        self.project_configurations = {'debug' : self.gen_id()}
+        self.project_configurations = {'debug': self.gen_id()}
 
     def generate_buildall_configurations_map(self):
-        self.buildall_configurations = {'debug' : self.gen_id()}
+        self.buildall_configurations = {'debug': self.gen_id()}
 
     def generate_test_configurations_map(self):
-        self.test_configurations = {'debug' : self.gen_id()}
+        self.test_configurations = {'debug': self.gen_id()}
 
     def generate_build_configurationlist_map(self):
         self.buildconflistmap = {}
@@ -184,35 +186,35 @@ class XCodeBackend(backends.Backend):
     def generate_pbx_aggregate_target(self):
         self.ofile.write('\n/* Begin PBXAggregateTarget section */\n')
         self.write_line('%s /* ALL_BUILD */ = {' % self.all_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXAggregateTarget;')
         self.write_line('buildConfigurationList = %s;' % self.all_buildconf_id)
         self.write_line('buildPhases = (')
         self.write_line(');')
         self.write_line('dependencies = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for t in self.build.targets:
             self.write_line('%s /* PBXTargetDependency */,' % self.pbx_dep_map[t])
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('name = ALL_BUILD;')
         self.write_line('productName = ALL_BUILD;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
         self.write_line('%s /* RUN_TESTS */ = {' % self.test_id)
-        self.indent_level +=1
+        self.indent_level += 1
         self.write_line('isa = PBXAggregateTarget;')
         self.write_line('buildConfigurationList = %s;' % self.test_buildconf_id)
         self.write_line('buildPhases = (')
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('%s /* test run command */,' % self.test_command_id)
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('dependencies = (')
         self.write_line(');')
         self.write_line('name = RUN_TESTS;')
         self.write_line('productName = RUN_TESTS;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
         self.ofile.write('/* End PBXAggregateTarget section */\n')
 
@@ -222,6 +224,9 @@ class XCodeBackend(backends.Backend):
         otempl = '%s /* %s */ = { isa = PBXBuildFile; fileRef = %s /* %s */;};\n'
         for t in self.build.targets.values():
             for s in t.sources:
+                if isinstance(s, mesonlib.File):
+                    s = s.fname
+
                 if isinstance(s, str):
                     s = os.path.join(t.subdir, s)
                     idval = self.buildmap[s]
@@ -250,7 +255,7 @@ class XCodeBackend(backends.Backend):
             self.write_line('COPY_PHASE_STRIP = NO;\n')
             self.indent_level -= 1
             self.write_line('};\n')
-            self.write_line('name = %s;\n' % name)
+            self.write_line('name = "%s";\n' % name)
             self.indent_level -= 1
             self.write_line('};\n')
         self.ofile.write('/* End PBXBuildStyle section */\n')
@@ -264,8 +269,8 @@ class XCodeBackend(backends.Backend):
             self.write_line('containerPortal = %s /* Project object */;' % self.project_uid)
             self.write_line('proxyType = 1;')
             self.write_line('remoteGlobalIDString = %s;' % self.native_targets[t])
-            self.write_line('remoteInfo = %s;' % t)
-            self.indent_level-=1
+            self.write_line('remoteInfo = "%s";' % t)
+            self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End PBXContainerItemProxy section */\n')
 
@@ -285,12 +290,10 @@ class XCodeBackend(backends.Backend):
             reftype = 0
             if isinstance(t, build.Executable):
                 typestr = 'compiled.mach-o.executable'
-                path = t.get_filename()
+                path = fname
             elif isinstance(t, build.SharedLibrary):
-                # OSX has a completely different shared library
-                # naming scheme so do this manually.
                 typestr = self.get_xcodetype('dummy.dylib')
-                path = t.get_osx_filename()
+                path = fname
             else:
                 typestr = self.get_xcodetype(fname)
                 path = '"%s"' % t.get_filename()
@@ -308,14 +311,14 @@ class XCodeBackend(backends.Backend):
         resources_id = self.gen_id()
         products_id = self.gen_id()
         self.write_line('%s = {' % self.maingroup_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXGroup;')
         self.write_line('children = (')
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('%s /* Sources */,' % sources_id)
         self.write_line('%s /* Resources */,' % resources_id)
         self.write_line('%s /* Products */,' % products_id)
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('sourceTree = "<group>";')
         self.indent_level -= 1
@@ -323,48 +326,48 @@ class XCodeBackend(backends.Backend):
 
         # Sources
         self.write_line('%s /* Sources */ = {' % sources_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXGroup;')
         self.write_line('children = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for t in self.build.targets:
             self.write_line('%s /* %s */,' % (groupmap[t], t))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('name = Sources;')
         self.write_line('sourcetree = "<group>";')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         self.write_line('%s /* Resources */ = {' % resources_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXGroup;')
         self.write_line('children = (')
         self.write_line(');')
         self.write_line('name = Resources;')
         self.write_line('sourceTree = "<group>";')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         # Targets
         for t in self.build.targets:
             self.write_line('%s /* %s */ = {' % (groupmap[t], t))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = PBXGroup;')
             self.write_line('children = (')
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('%s /* Source files */,' % target_src_map[t])
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line(');')
-            self.write_line('name = %s;' % t)
+            self.write_line('name = "%s";' % t)
             self.write_line('sourceTree = "<group>";')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
             self.write_line('%s /* Source files */ = {' % target_src_map[t])
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = PBXGroup;')
             self.write_line('children = (')
-            self.indent_level+=1
+            self.indent_level += 1
             for s in self.build.targets[t].sources:
                 s = os.path.join(s.subdir, s.fname)
                 if isinstance(s, str):
@@ -372,26 +375,26 @@ class XCodeBackend(backends.Backend):
             for o in self.build.targets[t].objects:
                 o = os.path.join(self.build.targets[t].subdir, o)
                 self.write_line('%s /* %s */,' % (self.filemap[o], o))
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line(');')
             self.write_line('name = "Source files";')
             self.write_line('sourceTree = "<group>";')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
 
         # And finally products
         self.write_line('%s /* Products */ = {' % products_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXGroup;')
         self.write_line('children = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for t in self.build.targets:
             self.write_line('%s /* %s */,' % (self.target_filemap[t], t))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('name = Products;')
         self.write_line('sourceTree = "<group>";')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
         self.ofile.write('/* End PBXGroup section */\n')
 
@@ -400,28 +403,28 @@ class XCodeBackend(backends.Backend):
         for tname, idval in self.native_targets.items():
             t = self.build.targets[tname]
             self.write_line('%s /* %s */ = {' % (idval, tname))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = PBXNativeTarget;')
-            self.write_line('buildConfigurationList = %s /* Build configuration list for PBXNativeTarget "%s" */;'\
+            self.write_line('buildConfigurationList = %s /* Build configuration list for PBXNativeTarget "%s" */;'
                             % (self.buildconflistmap[tname], tname))
             self.write_line('buildPhases = (')
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('%s /* Sources */,' % self.buildphasemap[tname])
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line(');')
             self.write_line('buildRules = (')
             self.write_line(');')
             self.write_line('dependencies = (')
-            self.indent_level+=1
+            self.indent_level += 1
             for lt in self.build.targets[tname].link_targets:
                 # NOT DOCUMENTED, may need to make different links
                 # to same target have different targetdependency item.
-                idval = self.pbx_dep_map[lt.get_basename()]
+                idval = self.pbx_dep_map[lt.get_id()]
                 self.write_line('%s /* PBXTargetDependency */,' % idval)
-            self.indent_level -=1
+            self.indent_level -= 1
             self.write_line(");")
-            self.write_line('name = %s;' % tname)
-            self.write_line('productName = %s;' % tname)
+            self.write_line('name = "%s";' % tname)
+            self.write_line('productName = "%s";' % tname)
             self.write_line('productReference = %s /* %s */;' % (self.target_filemap[tname], tname))
             if isinstance(t, build.Executable):
                 typestr = 'com.apple.product-type.tool'
@@ -432,7 +435,7 @@ class XCodeBackend(backends.Backend):
             else:
                 raise MesonException('Unknown target type for %s' % tname)
             self.write_line('productType = "%s";' % typestr)
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End PBXNativeTarget section */\n')
 
@@ -473,7 +476,7 @@ class XCodeBackend(backends.Backend):
         self.write_line('};')
         self.ofile.write('/* End PBXProject section */\n')
 
-    def generate_pbx_shell_build_phase(self):
+    def generate_pbx_shell_build_phase(self, test_data):
         self.ofile.write('\n/* Begin PBXShellScriptBuildPhase section */\n')
         self.write_line('%s = {' % self.test_command_id)
         self.indent_level += 1
@@ -489,12 +492,11 @@ class XCodeBackend(backends.Backend):
         self.write_line('shellPath = /bin/sh;')
         script_root = self.environment.get_script_dir()
         test_script = os.path.join(script_root, 'meson_test.py')
-        test_data = os.path.join(self.environment.get_scratch_dir(), 'meson_test_setup.dat')
         cmd = [sys.executable, test_script, test_data, '--wd', self.environment.get_build_dir()]
         cmdstr = ' '.join(["'%s'" % i for i in cmd])
         self.write_line('shellScript = "%s";' % cmdstr)
         self.write_line('showEnvVarsInLog = 0;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
         self.ofile.write('/* End PBXShellScriptBuildPhase section */\n')
 
@@ -502,19 +504,19 @@ class XCodeBackend(backends.Backend):
         self.ofile.write('\n/* Begin PBXSourcesBuildPhase section */\n')
         for name, phase_id in self.source_phase.items():
             self.write_line('%s /* Sources */ = {' % self.buildphasemap[name])
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = PBXSourcesBuildPhase;')
             self.write_line('buildActionMask = 2147483647;')
             self.write_line('files = (')
-            self.indent_level+=1
+            self.indent_level += 1
             for s in self.build.targets[name].sources:
                 s = os.path.join(s.subdir, s.fname)
                 if not self.environment.is_header(s):
                     self.write_line('%s /* %s */,' % (self.buildmap[s], os.path.join(self.environment.get_source_dir(), s)))
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line(');')
             self.write_line('runOnlyForDeploymentPostprocessing = 0;')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End PBXSourcesBuildPhase section */\n')
 
@@ -527,7 +529,7 @@ class XCodeBackend(backends.Backend):
             self.write_line('isa = PBXTargetDependency;')
             self.write_line('target = %s /* %s */;' % (self.native_targets[t], t))
             self.write_line('targetProxy = %s /* PBXContainerItemProxy */;' % self.containerproxy_map[t])
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End PBXTargetDependency section */\n')
 
@@ -536,24 +538,24 @@ class XCodeBackend(backends.Backend):
         # First the setup for the toplevel project.
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */ = {' % (self.project_configurations[buildtype], buildtype))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = XCBuildConfiguration;')
             self.write_line('buildSettings = {')
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('ARCHS = "$(ARCHS_STANDARD_32_64_BIT)";')
             self.write_line('ONLY_ACTIVE_ARCH = YES;')
-            self.write_line('SDKROOT = "/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.9.sdk";')
+            self.write_line('SDKROOT = "macosx";')
             self.write_line('SYMROOT = "%s/build";' % self.environment.get_build_dir())
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
-            self.write_line('name = %s;' % buildtype)
-            self.indent_level-=1
+            self.write_line('name = "%s";' % buildtype)
+            self.indent_level -= 1
             self.write_line('};')
 
         # Then the all target.
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */ = {' % (self.buildall_configurations[buildtype], buildtype))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = XCBuildConfiguration;')
             self.write_line('buildSettings = {')
             self.indent_level += 1
@@ -572,16 +574,16 @@ class XCodeBackend(backends.Backend):
             self.write_line('SYMROOT = "%s";' % self.environment.get_build_dir())
             self.write_line('USE_HEADERMAP = NO;')
             self.write_line('WARNING_CFLAGS = ("-Wmost", "-Wno-four-char-constants", "-Wno-unknown-pragmas", );')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
-            self.write_line('name = %s;' % buildtype)
-            self.indent_level-=1
+            self.write_line('name = "%s";' % buildtype)
+            self.indent_level -= 1
             self.write_line('};')
 
         # Then the test target.
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */ = {' % (self.test_configurations[buildtype], buildtype))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = XCBuildConfiguration;')
             self.write_line('buildSettings = {')
             self.indent_level += 1
@@ -600,14 +602,14 @@ class XCodeBackend(backends.Backend):
             self.write_line('SYMROOT = "%s";' % self.environment.get_build_dir())
             self.write_line('USE_HEADERMAP = NO;')
             self.write_line('WARNING_CFLAGS = ("-Wmost", "-Wno-four-char-constants", "-Wno-unknown-pragmas", );')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
-            self.write_line('name = %s;' % buildtype)
-            self.indent_level-=1
+            self.write_line('name = "%s";' % buildtype)
+            self.indent_level -= 1
             self.write_line('};')
 
         # Now finally targets.
-        langnamemap = {'c' : 'C', 'cpp' : 'CPLUSPLUS', 'objc' : 'OBJC', 'objcpp' : 'OBJCPLUSPLUS'}
+        langnamemap = {'c': 'C', 'cpp': 'CPLUSPLUS', 'objc': 'OBJC', 'objcpp': 'OBJCPLUSPLUS'}
         for target_name, target in self.build.targets.items():
             for buildtype in self.buildtypes:
                 dep_libs = []
@@ -620,7 +622,7 @@ class XCodeBackend(backends.Backend):
                         headerdirs.append(os.path.join(self.environment.get_build_dir(), cd))
                 for l in target.link_targets:
                     abs_path = os.path.join(self.environment.get_build_dir(),
-                                            l.subdir, buildtype, l.get_osx_filename())
+                                            l.subdir, buildtype, l.get_filename())
                     dep_libs.append("'%s'" % abs_path)
                     if isinstance(l, build.SharedLibrary):
                         links_dylib = True
@@ -635,9 +637,9 @@ class XCodeBackend(backends.Backend):
                     ldargs = dep_libs
                     install_path = ''
                 if dylib_version is not None:
-                    product_name = target_name + '.' + dylib_version
+                    product_name = target.get_basename() + '.' + dylib_version
                 else:
-                    product_name = target_name
+                    product_name = target.get_basename()
                 ldargs += target.link_args
                 ldstr = ' '.join(ldargs)
                 valid = self.buildconfmap[target_name][buildtype]
@@ -652,7 +654,7 @@ class XCodeBackend(backends.Backend):
                         langargs[langnamemap[lang]] = args
                 symroot = os.path.join(self.environment.get_build_dir(), target.subdir)
                 self.write_line('%s /* %s */ = {' % (valid, buildtype))
-                self.indent_level+=1
+                self.indent_level += 1
                 self.write_line('isa = XCBuildConfiguration;')
                 self.write_line('buildSettings = {')
                 self.indent_level += 1
@@ -687,57 +689,57 @@ class XCodeBackend(backends.Backend):
                 self.write_line('SYMROOT = "%s";' % symroot)
                 self.write_line('USE_HEADERMAP = NO;')
                 self.write_line('WARNING_CFLAGS = ("-Wmost", "-Wno-four-char-constants", "-Wno-unknown-pragmas", );')
-                self.indent_level-=1
+                self.indent_level -= 1
                 self.write_line('};')
-                self.write_line('name = %s;' % buildtype)
-                self.indent_level-=1
+                self.write_line('name = "%s";' % buildtype)
+                self.indent_level -= 1
                 self.write_line('};')
         self.ofile.write('/* End XCBuildConfiguration section */\n')
 
     def generate_xc_configurationList(self):
         self.ofile.write('\n/* Begin XCConfigurationList section */\n')
         self.write_line('%s /* Build configuration list for PBXProject "%s" */ = {' % (self.project_conflist, self.build.project_name))
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = XCConfigurationList;')
         self.write_line('buildConfigurations = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */,' % (self.project_configurations[buildtype], buildtype))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('defaultConfigurationIsVisible = 0;')
         self.write_line('defaultConfigurationName = debug;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         # Now the all target
         self.write_line('%s /* Build configuration list for PBXAggregateTarget "ALL_BUILD" */ = {' % self.all_buildconf_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = XCConfigurationList;')
         self.write_line('buildConfigurations = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */,' % (self.buildall_configurations[buildtype], buildtype))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('defaultConfigurationIsVisible = 0;')
         self.write_line('defaultConfigurationName = debug;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         # Test target
         self.write_line('%s /* Build configuration list for PBXAggregateTarget "ALL_BUILD" */ = {' % self.test_buildconf_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = XCConfigurationList;')
         self.write_line('buildConfigurations = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */,' % (self.test_configurations[buildtype], buildtype))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('defaultConfigurationIsVisible = 0;')
         self.write_line('defaultConfigurationName = debug;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         for target_name in self.build.targets:
@@ -753,7 +755,7 @@ class XCodeBackend(backends.Backend):
             self.indent_level -= 1
             self.write_line(');')
             self.write_line('defaultConfigurationIsVisible = 0;')
-            self.write_line('defaultConfigurationName = %s;' % typestr)
+            self.write_line('defaultConfigurationName = "%s";' % typestr)
             self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End XCConfigurationList section */\n')
