@@ -13,10 +13,14 @@
 # limitations under the License.
 
 import pickle, os, uuid
-from .mesonlib import MesonException, default_libdir, default_libexecdir, default_prefix
+from pathlib import PurePath
+from collections import OrderedDict
+from .mesonlib import MesonException, commonpath
+from .mesonlib import default_libdir, default_libexecdir, default_prefix
+import ast
 
-version = '0.38.0'
-backendlist = ['ninja', 'vs2010', 'vs2015', 'xcode']
+version = '0.40.1'
+backendlist = ['ninja', 'vs', 'vs2010', 'vs2015', 'vs2017', 'xcode']
 
 class UserOption:
     def __init__(self, name, description, choices):
@@ -28,6 +32,12 @@ class UserOption:
     def parse_string(self, valuestring):
         return valuestring
 
+    # Check that the input is a valid value and return the
+    # "cleaned" or "native" version. For example the Boolean
+    # option could take the string "true" and return True.
+    def validate_value(self, value):
+        raise RuntimeError('Derived option class did not override validate_value.')
+
 class UserStringOption(UserOption):
     def __init__(self, name, description, value, choices=None):
         super().__init__(name, description, choices)
@@ -41,6 +51,10 @@ class UserStringOption(UserOption):
         self.validate(newvalue)
         self.value = newvalue
 
+    def validate_value(self, value):
+        self.validate(value)
+        return value
+
 class UserBooleanOption(UserOption):
     def __init__(self, name, description, value):
         super().__init__(name, description, [True, False])
@@ -68,6 +82,9 @@ class UserBooleanOption(UserOption):
     def __bool__(self):
         return self.value
 
+    def validate_value(self, value):
+        return self.tobool(value)
+
 class UserComboOption(UserOption):
     def __init__(self, name, description, choices, value):
         super().__init__(name, description, choices)
@@ -84,22 +101,36 @@ class UserComboOption(UserOption):
             raise MesonException('Value "%s" for combo option "%s" is not one of the choices. Possible choices are: %s.' % (newvalue, self.name, optionsstring))
         self.value = newvalue
 
+    def validate_value(self, value):
+        if value not in self.choices:
+            raise MesonException('Value %s not one of accepted values.' % value)
+        return value
+
 class UserStringArrayOption(UserOption):
     def __init__(self, name, description, value, **kwargs):
         super().__init__(name, description, kwargs.get('choices', []))
         self.set_value(value)
 
-    def set_value(self, newvalue):
-        if isinstance(newvalue, str):
-            if not newvalue.startswith('['):
-                raise MesonException('Valuestring does not define an array: ' + newvalue)
-            newvalue = eval(newvalue, {}, {}) # Yes, it is unsafe.
+    def validate(self, value):
+        if isinstance(value, str):
+            if not value.startswith('['):
+                raise MesonException('Valuestring does not define an array: ' + value)
+            newvalue = ast.literal_eval(value)
+        else:
+            newvalue = value
         if not isinstance(newvalue, list):
             raise MesonException('"{0}" should be a string array, but it is not'.format(str(newvalue)))
         for i in newvalue:
             if not isinstance(i, str):
                 raise MesonException('String array element "{0}" is not a string.'.format(str(newvalue)))
-        self.value = newvalue
+        return newvalue
+
+    def set_value(self, newvalue):
+        self.value = self.validate(newvalue)
+
+    def validate_value(self, value):
+        self.validate(value)
+        return value
 
 # This class contains all data that must persist over multiple
 # invocations of Meson. It is roughly the same thing as
@@ -117,17 +148,18 @@ class CoreData:
         self.user_options = {}
         self.compiler_options = {}
         self.base_options = {}
-        # These two, external_*args, are set via env vars CFLAGS, LDFLAGS, etc
+        # These external_*args, are set via env vars CFLAGS, LDFLAGS, etc
         # but only when not cross-compiling.
-        self.external_args = {}
-        self.external_link_args = {}
+        self.external_preprocess_args = {} # CPPFLAGS only
+        self.external_args = {} # CPPFLAGS + CFLAGS
+        self.external_link_args = {} # CFLAGS + LDFLAGS (with MSVC: only LDFLAGS)
         if options.cross_file is not None:
             self.cross_file = os.path.join(os.getcwd(), options.cross_file)
         else:
             self.cross_file = None
-
-        self.compilers = {}
-        self.cross_compilers = {}
+        self.wrap_mode = options.wrap_mode
+        self.compilers = OrderedDict()
+        self.cross_compilers = OrderedDict()
         self.deps = {}
         self.modules = {}
         # Only to print a warning if it changes between Meson invocations.
@@ -158,7 +190,10 @@ class CoreData:
         if option.endswith('dir') and os.path.isabs(value) and \
            option not in builtin_dir_noprefix_options:
             # Value must be a subdir of the prefix
-            if os.path.commonpath([value, prefix]) != prefix:
+            # commonpath will always return a path in the native format, so we
+            # must use pathlib.PurePath to do the same conversion before
+            # comparing.
+            if commonpath([value, prefix]) != str(PurePath(prefix)):
                 m = 'The value of the {!r} option is {!r} which must be a ' \
                     'subdir of the prefix {!r}.\nNote that if you pass a ' \
                     'relative path, it is assumed to be a subdir of prefix.'
@@ -194,9 +229,16 @@ class CoreData:
         elif optname in self.builtins:
             prefix = self.builtins['prefix'].value
             value = self.sanitize_dir_option_value(prefix, optname, value)
-            self.builtins[optname].set_value(value)
         else:
             raise RuntimeError('Tried to set unknown builtin option %s.' % optname)
+        self.builtins[optname].set_value(value)
+
+    def validate_option_value(self, option_name, override_value):
+        for opts in (self.builtins, self.base_options, self.compiler_options, self.user_options):
+            if option_name in opts:
+                opt = opts[option_name]
+                return opt.validate_value(override_value)
+        raise MesonException('Tried to validate unknown option %s.' % option_name)
 
 def load(filename):
     load_fail_msg = 'Coredata file {!r} is corrupted. Try with a fresh build tree.'.format(filename)
@@ -253,7 +295,7 @@ def get_builtin_option_default(optname):
 builtin_options = {
     'buildtype':  [UserComboOption, 'Build type to use.', ['plain', 'debug', 'debugoptimized', 'release', 'minsize'], 'debug'],
     'strip':      [UserBooleanOption, 'Strip targets on install.', False],
-    'unity':      [UserBooleanOption, 'Unity build.', False],
+    'unity':      [UserComboOption, 'Unity build.', ['on', 'off', 'subprojects'], 'off'],
     'prefix':     [UserStringOption, 'Installation prefix.', default_prefix()],
     'libdir':     [UserStringOption, 'Library directory.', default_libdir()],
     'libexecdir': [UserStringOption, 'Library executable directory.', default_libexecdir()],
@@ -299,7 +341,6 @@ forbidden_target_names = {'clean': None,
                           'PHONY': None,
                           'all': None,
                           'test': None,
-                          'test:': None,
                           'benchmark': None,
                           'install': None,
                           'uninstall': None,
