@@ -1,4 +1,4 @@
-# Copyright 2016 The Meson development team
+# Copyright 2016-2017 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -65,7 +65,7 @@ class InvalidCode(InterpreterException):
 class InvalidArguments(InterpreterException):
     pass
 
-class InterpreterObject():
+class InterpreterObject:
     def __init__(self):
         self.methods = {}
 
@@ -86,6 +86,7 @@ class InterpreterBase:
         self.builtin = {}
         self.subdir = subdir
         self.variables = {}
+        self.argument_depth = 0
 
     def load_root_meson_file(self):
         mesonfile = os.path.join(self.source_root, self.subdir, environment.build_filename)
@@ -391,6 +392,8 @@ class InterpreterBase:
             return self.int_method_call(obj, method_name, args)
         if isinstance(obj, list):
             return self.array_method_call(obj, method_name, self.reduce_arguments(args)[0])
+        if isinstance(obj, mesonlib.File):
+            raise InvalidArguments('File object "%s" is not callable.' % obj)
         if not isinstance(obj, InterpreterObject):
             raise InvalidArguments('Variable "%s" is not callable.' % object_name)
         (args, kwargs) = self.reduce_arguments(args)
@@ -403,19 +406,19 @@ class InterpreterBase:
         (posargs, _) = self.reduce_arguments(args)
         if method_name == 'to_string':
             if len(posargs) == 0:
-                if obj == True:
+                if obj:
                     return 'true'
                 else:
                     return 'false'
             elif len(posargs) == 2 and isinstance(posargs[0], str) and isinstance(posargs[1], str):
-                if obj == True:
+                if obj:
                     return posargs[0]
                 else:
                     return posargs[1]
             else:
                 raise InterpreterException('bool.to_string() must have either no arguments or exactly two string arguments that signify what values to return for true and false.')
         elif method_name == 'to_int':
-            if obj == True:
+            if obj:
                 return 1
             else:
                 return 0
@@ -500,18 +503,30 @@ class InterpreterBase:
             return len(obj)
         elif method_name == 'get':
             index = args[0]
+            fallback = None
+            if len(args) == 2:
+                fallback = args[1]
+            elif len(args) > 2:
+                m = 'Array method \'get()\' only takes two arguments: the ' \
+                    'index and an optional fallback value if the index is ' \
+                    'out of range.'
+                raise InvalidArguments(m)
             if not isinstance(index, int):
                 raise InvalidArguments('Array index must be a number.')
             if index < -len(obj) or index >= len(obj):
-                raise InvalidArguments('Array index %s is out of bounds for array of size %d.' % (index, len(obj)))
+                if fallback is None:
+                    m = 'Array index {!r} is out of bounds for array of size {!r}.'
+                    raise InvalidArguments(m.format(index, len(obj)))
+                return fallback
             return obj[index]
-        raise InterpreterException('Arrays do not have a method called "%s".' % method_name)
-
+        m = 'Arrays do not have a method called {!r}.'
+        raise InterpreterException(m.format(method_name))
 
     def reduce_arguments(self, args):
         assert(isinstance(args, mparser.ArgumentNode))
         if args.incorrect_order():
             raise InvalidArguments('All keyword arguments must be after positional arguments.')
+        self.argument_depth += 1
         reduced_pos = [self.evaluate_statement(arg) for arg in args.arguments]
         reduced_kw = {}
         for key in args.kwargs.keys():
@@ -521,7 +536,8 @@ class InterpreterBase:
             reduced_kw[key] = self.evaluate_statement(a)
         if not isinstance(reduced_pos, list):
             reduced_pos = [reduced_pos]
-        return (reduced_pos, reduced_kw)
+        self.argument_depth -= 1
+        return reduced_pos, reduced_kw
 
     def flatten(self, args):
         if isinstance(args, mparser.StringNode):
@@ -541,6 +557,9 @@ class InterpreterBase:
 
     def assignment(self, node):
         assert(isinstance(node, mparser.AssignmentNode))
+        if self.argument_depth != 0:
+            raise InvalidArguments('''Tried to assign values inside an argument list.
+To specify a keyword argument, use : instead of =.''')
         var_name = node.var_name
         if not isinstance(var_name, str):
             raise InvalidArguments('Tried to assign value to a non-variable.')
@@ -552,7 +571,7 @@ class InterpreterBase:
         if isinstance(value, MutableInterpreterObject):
             value = copy.deepcopy(value)
         self.set_variable(var_name, value)
-        return value
+        return None
 
     def set_variable(self, varname, variable):
         if variable is None:
@@ -610,7 +629,7 @@ class InterpreterBase:
 
 #    @noKwargs
     def func_get_variable(self, node, args, kwargs):
-        if len(args)<1 or len(args)>2:
+        if len(args) < 1 or len(args) > 2:
             raise InvalidCode('Get_variable takes one or two arguments.')
         varname = args[0]
         if not isinstance(varname, str):
@@ -633,4 +652,3 @@ class InterpreterBase:
 
     def is_elementary_type(self, v):
         return isinstance(v, (int, float, str, bool, list))
-
