@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-
 # Copyright 2012-2016 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,76 +13,74 @@
 # limitations under the License.
 
 import sys, stat, traceback, pickle, argparse
-import datetime
+import time, datetime
 import os.path
 from . import environment, interpreter, mesonlib
 from . import build
 import platform
 from . import mlog, coredata
+from .mesonlib import MesonException
+from .wrap import WrapMode
 
-from .coredata import MesonException, build_types, layouts, warning_levels, libtypelist
-
-backendlist = ['ninja', 'vs2010', 'xcode']
 
 parser = argparse.ArgumentParser()
 
 default_warning = '1'
 
-if mesonlib.is_windows():
-    def_prefix = 'c:/'
-else:
-    def_prefix = '/usr/local'
+def add_builtin_argument(name, **kwargs):
+    k = kwargs.get('dest', name.replace('-', '_'))
+    c = coredata.get_builtin_option_choices(k)
+    b = True if kwargs.get('action', None) in ['store_true', 'store_false'] else False
+    h = coredata.get_builtin_option_description(k)
+    if not b:
+        h = h.rstrip('.') + ' (default: %s).' % coredata.get_builtin_option_default(k)
+    if c and not b:
+        kwargs['choices'] = c
+    parser.add_argument('--' + name, default=coredata.get_builtin_option_default(k), help=h, **kwargs)
 
-parser.add_argument('--prefix', default=def_prefix, dest='prefix',
-                    help='the installation prefix (default: %(default)s)')
-parser.add_argument('--libdir', default=mesonlib.default_libdir(), dest='libdir',
-                    help='the installation subdir of libraries (default: %(default)s)')
-parser.add_argument('--bindir', default='bin', dest='bindir',
-                    help='the installation subdir of executables (default: %(default)s)')
-parser.add_argument('--includedir', default='include', dest='includedir',
-                    help='relative path of installed headers (default: %(default)s)')
-parser.add_argument('--datadir', default='share', dest='datadir',
-                    help='relative path to the top of data file subdirectory (default: %(default)s)')
-parser.add_argument('--mandir', default='share/man', dest='mandir',
-                    help='relative path of man files (default: %(default)s)')
-parser.add_argument('--localedir', default='share/locale', dest='localedir',
-                    help='relative path of locale data (default: %(default)s)')
-parser.add_argument('--backend', default='ninja', dest='backend', choices=backendlist,
-                    help='backend to use (default: %(default)s)')
-parser.add_argument('--buildtype', default='debug', choices=build_types, dest='buildtype',
-                    help='build type go use (default: %(default)s)')
-parser.add_argument('--strip', action='store_true', dest='strip', default=False,\
-                    help='strip targets on install (default: %(default)s)')
-parser.add_argument('--enable-gcov', action='store_true', dest='coverage', default=False,\
-                    help='measure test coverage')
-parser.add_argument('--disable-pch', action='store_false', dest='use_pch', default=True,\
-                    help='do not use precompiled headers')
-parser.add_argument('--unity', action='store_true', dest='unity', default=False,\
-                    help='unity build')
-parser.add_argument('--werror', action='store_true', dest='werror', default=False,\
-                    help='Treat warnings as errors')
-parser.add_argument('--layout', choices=layouts, dest='layout', default='mirror',\
-                    help='Build directory layout.')
-parser.add_argument('--default-library', choices=libtypelist, dest='default_library',
-                    default='shared', help='Default library type.')
-parser.add_argument('--warnlevel', default=default_warning, dest='warning_level', choices=warning_levels,\
-                    help='Level of compiler warnings to use (larger is more, default is %(default)s)')
-parser.add_argument('--cross-file', default=None, dest='cross_file',
-                    help='file describing cross compilation environment')
+add_builtin_argument('prefix')
+add_builtin_argument('libdir')
+add_builtin_argument('libexecdir')
+add_builtin_argument('bindir')
+add_builtin_argument('sbindir')
+add_builtin_argument('includedir')
+add_builtin_argument('datadir')
+add_builtin_argument('mandir')
+add_builtin_argument('infodir')
+add_builtin_argument('localedir')
+add_builtin_argument('sysconfdir')
+add_builtin_argument('localstatedir')
+add_builtin_argument('sharedstatedir')
+add_builtin_argument('backend')
+add_builtin_argument('buildtype')
+add_builtin_argument('strip', action='store_true')
+add_builtin_argument('unity')
+add_builtin_argument('werror', action='store_true')
+add_builtin_argument('layout')
+add_builtin_argument('default-library')
+add_builtin_argument('warnlevel', dest='warning_level')
+add_builtin_argument('stdsplit', action='store_false')
+add_builtin_argument('errorlogs', action='store_false')
+
+parser.add_argument('--cross-file', default=None,
+                    help='File describing cross compilation environment.')
 parser.add_argument('-D', action='append', dest='projectoptions', default=[],
                     help='Set project options.')
-parser.add_argument('-v', '--version', action='store_true', dest='print_version', default=False,
-                    help='Print version.')
+parser.add_argument('-v', '--version', action='version',
+                    version=coredata.version)
+                    # See the mesonlib.WrapMode enum for documentation
+parser.add_argument('--wrap-mode', default=WrapMode.default,
+                    type=lambda t: getattr(WrapMode, t), choices=WrapMode,
+                    help='Special wrap mode to use')
 parser.add_argument('directories', nargs='*')
 
-class MesonApp():
+class MesonApp:
 
-    def __init__(self, dir1, dir2, script_file, handshake, options):
+    def __init__(self, dir1, dir2, script_launcher, handshake, options, original_cmd_line_args):
         (self.source_dir, self.build_dir) = self.validate_dirs(dir1, dir2, handshake)
-        if not os.path.isabs(options.prefix):
-            raise RuntimeError('--prefix must be an absolute path.')
-        self.meson_script_file = script_file
+        self.meson_script_launcher = script_launcher
         self.options = options
+        self.original_cmd_line_args = original_cmd_line_args
 
     def has_build_file(self, dirname):
         fname = os.path.join(dirname, environment.build_filename)
@@ -93,6 +89,10 @@ class MesonApp():
     def validate_core_dirs(self, dir1, dir2):
         ndir1 = os.path.abspath(dir1)
         ndir2 = os.path.abspath(dir2)
+        if not os.path.exists(ndir1):
+            os.makedirs(ndir1)
+        if not os.path.exists(ndir2):
+            os.makedirs(ndir2)
         if not stat.S_ISDIR(os.stat(ndir1).st_mode):
             raise RuntimeError('%s is not a directory' % dir1)
         if not stat.S_ISDIR(os.stat(ndir2).st_mode):
@@ -102,9 +102,9 @@ class MesonApp():
         if self.has_build_file(ndir1):
             if self.has_build_file(ndir2):
                 raise RuntimeError('Both directories contain a build file %s.' % environment.build_filename)
-            return (ndir1, ndir2)
+            return ndir1, ndir2
         if self.has_build_file(ndir2):
-            return (ndir2, ndir1)
+            return ndir2, ndir1
         raise RuntimeError('Neither directory contains a build file %s.' % environment.build_filename)
 
     def validate_dirs(self, dir1, dir2, handshake):
@@ -115,20 +115,30 @@ class MesonApp():
                 msg = '''Trying to run Meson on a build directory that has already been configured.
 If you want to build it, just run your build command (e.g. ninja) inside the
 build directory. Meson will autodetect any changes in your setup and regenerate
-itself as required.'''
+itself as required.
+
+If you want to change option values, use the mesonconf tool instead.'''
                 raise RuntimeError(msg)
         else:
             if handshake:
                 raise RuntimeError('Something went terribly wrong. Please file a bug.')
-        return (src_dir, build_dir)
+        return src_dir, build_dir
+
+    def check_pkgconfig_envvar(self, env):
+        curvar = os.environ.get('PKG_CONFIG_PATH', '')
+        if curvar != env.coredata.pkgconf_envvar:
+            mlog.warning('PKG_CONFIG_PATH has changed between invocations from "%s" to "%s".' %
+                         (env.coredata.pkgconf_envvar, curvar))
+            env.coredata.pkgconf_envvar = curvar
 
     def generate(self):
-        env = environment.Environment(self.source_dir, self.build_dir, self.meson_script_file, self.options)
+        env = environment.Environment(self.source_dir, self.build_dir, self.meson_script_launcher, self.options, self.original_cmd_line_args)
         mlog.initialize(env.get_log_dir())
         mlog.debug('Build started at', datetime.datetime.now().isoformat())
         mlog.debug('Python binary:', sys.executable)
         mlog.debug('Python system:', platform.system())
         mlog.log(mlog.bold('The Meson build system'))
+        self.check_pkgconfig_envvar(env)
         mlog.log('Version:', coredata.version)
         mlog.log('Source dir:', mlog.bold(self.source_dir))
         mlog.log('Build dir:', mlog.bold(self.build_dir))
@@ -140,9 +150,19 @@ itself as required.'''
         if self.options.backend == 'ninja':
             from .backend import ninjabackend
             g = ninjabackend.NinjaBackend(b)
+        elif self.options.backend == 'vs':
+            from .backend import vs2010backend
+            g = vs2010backend.autodetect_vs_version(b)
+            mlog.log('Auto detected Visual Studio backend:', mlog.bold(g.name))
         elif self.options.backend == 'vs2010':
             from .backend import vs2010backend
             g = vs2010backend.Vs2010Backend(b)
+        elif self.options.backend == 'vs2015':
+            from .backend import vs2015backend
+            g = vs2015backend.Vs2015Backend(b)
+        elif self.options.backend == 'vs2017':
+            from .backend import vs2017backend
+            g = vs2017backend.Vs2017Backend(b)
         elif self.options.backend == 'xcode':
             from .backend import xcodebackend
             g = xcodebackend.XCodeBackend(b)
@@ -158,19 +178,31 @@ itself as required.'''
         mlog.log('Build machine cpu family:', mlog.bold(intr.builtin['build_machine'].cpu_family_method([], {})))
         mlog.log('Build machine cpu:', mlog.bold(intr.builtin['build_machine'].cpu_method([], {})))
         intr.run()
+        coredata_mtime = time.time()
         g.generate(intr)
-        env.generating_finished()
+        g.run_postconf_scripts()
         dumpfile = os.path.join(env.get_scratch_dir(), 'build.dat')
-        pickle.dump(b, open(dumpfile, 'wb'))
+        with open(dumpfile, 'wb') as f:
+            pickle.dump(b, f)
+        # Write this last since we use the existence of this file to check if
+        # we generated the build file successfully, so we don't want an error
+        # that pops up during generation, post-conf scripts, etc to cause us to
+        # incorrectly signal a successful meson run which will cause an error
+        # about an already-configured build directory when the user tries again.
+        #
+        # However, we set the mtime to an earlier value to ensure that doing an
+        # mtime comparison between the coredata dump and other build files
+        # shows the build files to be newer, not older.
+        env.dump_coredata(coredata_mtime)
 
 def run_script_command(args):
     cmdname = args[0]
     cmdargs = args[1:]
-    if cmdname == 'test':
-        import mesonbuild.scripts.meson_test as abc
+    if cmdname == 'exe':
+        import mesonbuild.scripts.meson_exe as abc
         cmdfunc = abc.run
-    elif cmdname == 'benchmark':
-        import mesonbuild.scripts.meson_benchmark as abc
+    elif cmdname == 'cleantrees':
+        import mesonbuild.scripts.cleantrees as abc
         cmdfunc = abc.run
     elif cmdname == 'install':
         import mesonbuild.scripts.meson_install as abc
@@ -196,9 +228,21 @@ def run_script_command(args):
     elif cmdname == 'symbolextractor':
         import mesonbuild.scripts.symbolextractor as abc
         cmdfunc = abc.run
+    elif cmdname == 'scanbuild':
+        import mesonbuild.scripts.scanbuild as abc
+        cmdfunc = abc.run
     elif cmdname == 'vcstagger':
         import mesonbuild.scripts.vcstagger as abc
         cmdfunc = abc.run
+    elif cmdname == 'gettext':
+        import mesonbuild.scripts.gettext as abc
+        cmdfunc = abc.run
+    elif cmdname == 'yelphelper':
+        import mesonbuild.scripts.yelphelper as abc
+        cmdfunc = abc.run
+    elif cmdname == 'uninstall':
+        import mesonbuild.scripts.uninstall as abc
+        cmdfunc = abc.run
     else:
         raise MesonException('Unknown internal command {}.'.format(cmdname))
     return cmdfunc(cmdargs)
@@ -211,35 +255,39 @@ def run(mainfile, args):
         return 1
     if len(args) >= 2 and args[0] == '--internal':
         if args[1] != 'regenerate':
-            sys.exit(run_script_command(args[1:]))
+            script = args[1]
+            try:
+                sys.exit(run_script_command(args[1:]))
+            except MesonException as e:
+                mlog.log(mlog.red('\nError in {} helper script:'.format(script)))
+                mlog.log(e)
+                sys.exit(1)
         args = args[2:]
         handshake = True
     else:
         handshake = False
     args = mesonlib.expand_arguments(args)
     options = parser.parse_args(args)
-    if options.print_version:
-        print(coredata.version)
-        return 0
     args = options.directories
     if len(args) == 0 or len(args) > 2:
-        print('{} <source directory> <build directory>'.format(sys.argv[0]))
-        print('If you omit either directory, the current directory is substituted.')
-        print('Run {} --help for more information.'.format(sys.argv[0]))
-        return 1
-    dir1 = args[0]
-    if len(args) > 1:
-        dir2 = args[1]
-    else:
-        dir2 = '.'
-    while os.path.islink(mainfile):
-        resolved = os.readlink(mainfile)
-        if resolved[0] != '/':
-            mainfile = os.path.join(os.path.dirname(mainfile), resolved)
+        # if there's a meson.build in the dir above, and not in the current
+        # directory, assume we're in the build directory
+        if len(args) == 0 and not os.path.exists('meson.build') and os.path.exists('../meson.build'):
+            dir1 = '..'
+            dir2 = '.'
         else:
-            mainfile = resolved
+            print('{} <source directory> <build directory>'.format(sys.argv[0]))
+            print('If you omit either directory, the current directory is substituted.')
+            print('Run {} --help for more information.'.format(sys.argv[0]))
+            return 1
+    else:
+        dir1 = args[0]
+        if len(args) > 1:
+            dir2 = args[1]
+        else:
+            dir2 = '.'
     try:
-        app = MesonApp(dir1, dir2, mainfile, handshake, options)
+        app = MesonApp(dir1, dir2, mainfile, handshake, options, sys.argv)
     except Exception as e:
         # Log directory does not exist, so just print
         # to stdout.
