@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-
 # Copyright 2014-2016 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,13 +20,16 @@ Currently only works for the Ninja backend. Others use generated
 project files and don't need this info."""
 
 import json, pickle
-from . import coredata, build, mesonlib
+from . import coredata, build
 import argparse
 import sys, os
+import pathlib
 
 parser = argparse.ArgumentParser()
 parser.add_argument('--targets', action='store_true', dest='list_targets', default=False,
                     help='List top level targets.')
+parser.add_argument('--installed', action='store_true', dest='list_installed', default=False,
+                    help='List all installed files and directories.')
 parser.add_argument('--target-files', action='store', dest='target_files', default=None,
                     help='List source files for a given target.')
 parser.add_argument('--buildsystem-files', action='store_true', dest='buildsystem_files', default=False,
@@ -40,10 +41,36 @@ parser.add_argument('--tests', action='store_true', dest='tests', default=False,
 parser.add_argument('--benchmarks', action='store_true', dest='benchmarks', default=False,
                     help='List all benchmarks.')
 parser.add_argument('--dependencies', action='store_true', dest='dependencies', default=False,
-                    help='list external dependencies.')
-parser.add_argument('args', nargs='+')
+                    help='List external dependencies.')
+parser.add_argument('--projectinfo', action='store_true', dest='projectinfo', default=False,
+                    help='Information about projects.')
+parser.add_argument('builddir', nargs='?', help='The build directory')
 
-def list_targets(coredata, builddata):
+def determine_installed_path(target, installdata):
+    install_target = None
+    for i in installdata.targets:
+        if os.path.split(i[0])[1] == target.get_filename(): # FIXME, might clash due to subprojects.
+            install_target = i
+            break
+    if install_target is None:
+        raise RuntimeError('Something weird happened. File a bug.')
+    fname = i[0]
+    outdir = i[1]
+    outname = os.path.join(installdata.prefix, outdir, os.path.split(fname)[-1])
+    # Normalize the path by using os.path.sep consistently, etc.
+    # Does not change the effective path.
+    return str(pathlib.PurePath(outname))
+
+
+def list_installed(installdata):
+    res = {}
+    if installdata is not None:
+        for path, installpath, unused_prefix in installdata.data:
+            res[path] = os.path.join(installdata.prefix, installpath)
+    print(json.dumps(res))
+
+
+def list_targets(coredata, builddata, installdata):
     tlist = []
     for (idname, target) in builddata.get_targets().items():
         t = {}
@@ -68,8 +95,9 @@ def list_targets(coredata, builddata):
         else:
             typename = 'unknown'
         t['type'] = typename
-        if target.should_install():
+        if installdata and target.should_install():
             t['installed'] = True
+            t['install_filename'] = determine_installed_path(target, installdata)
         else:
             t['installed'] = False
         tlist.append(t)
@@ -86,23 +114,11 @@ def list_target_files(target_name, coredata, builddata):
     print(json.dumps(sources))
 
 def list_buildoptions(coredata, builddata):
-    buildtype= {'choices': ['plain', 'debug', 'debugoptimized', 'release', 'minsize'],
-                'type' : 'combo',
-                'value' : coredata.get_builtin_option('buildtype'),
-                'description' : 'Build type',
-                'name' : 'type'}
-    strip = {'value' : coredata.get_builtin_option('strip'),
-             'type' : 'boolean',
-             'description' : 'Strip on install',
-             'name' : 'strip'}
-    unity = {'value' : coredata.get_builtin_option('unity'),
-             'type' : 'boolean',
-             'description' : 'Unity build',
-             'name' : 'unity'}
-    optlist = [buildtype, strip, unity]
+    optlist = []
     add_keys(optlist, coredata.user_options)
     add_keys(optlist, coredata.compiler_options)
     add_keys(optlist, coredata.base_options)
+    add_keys(optlist, coredata.builtins)
     print(json.dumps(optlist))
 
 def add_keys(optlist, options):
@@ -156,7 +172,10 @@ def list_tests(testdata):
         else:
             fname = t.fname
         to['cmd'] = fname + t.cmd_args
-        to['env'] = t.env
+        if isinstance(t.env, build.EnvironmentVariables):
+            to['env'] = t.env.get_env(os.environ)
+        else:
+            to['env'] = t.env
         to['name'] = t.name
         to['workdir'] = t.workdir
         to['timeout'] = t.timeout
@@ -164,29 +183,54 @@ def list_tests(testdata):
         result.append(to)
     print(json.dumps(result))
 
+def list_projinfo(builddata):
+    result = {}
+    result['name'] = builddata.project_name
+    result['version'] = builddata.project_version
+    subprojects = []
+    for k, v in builddata.subprojects.items():
+        c = {'name': k,
+             'version': v}
+        subprojects.append(c)
+    result['subprojects'] = subprojects
+    print(json.dumps(result))
+
 def run(args):
+    datadir = 'meson-private'
     options = parser.parse_args(args)
-    if len(options.args) > 1:
-        print('Too many arguments')
+    if options.builddir is not None:
+        datadir = os.path.join(options.builddir, datadir)
+    if not os.path.isdir(datadir):
+        print('Current directory is not a build dir. Please specify it or '
+              'change the working directory to it.')
         return 1
-    elif len(options.args) == 1:
-        bdir = options.args[0]
-    else:
-        bdir = ''
-    corefile = os.path.join(bdir, 'meson-private/coredata.dat')
-    buildfile = os.path.join(bdir, 'meson-private/build.dat')
-    testfile = os.path.join(bdir, 'meson-private/meson_test_setup.dat')
-    benchmarkfile = os.path.join(bdir, 'meson-private/meson_benchmark_setup.dat')
+
+    corefile = os.path.join(datadir, 'coredata.dat')
+    buildfile = os.path.join(datadir, 'build.dat')
+    installfile = os.path.join(datadir, 'install.dat')
+    testfile = os.path.join(datadir, 'meson_test_setup.dat')
+    benchmarkfile = os.path.join(datadir, 'meson_benchmark_setup.dat')
+
+    # Load all data files
     with open(corefile, 'rb') as f:
         coredata = pickle.load(f)
     with open(buildfile, 'rb') as f:
         builddata = pickle.load(f)
     with open(testfile, 'rb') as f:
-       testdata = pickle.load(f)
+        testdata = pickle.load(f)
     with open(benchmarkfile, 'rb') as f:
-       benchmarkdata = pickle.load(f)
+        benchmarkdata = pickle.load(f)
+    # Install data is only available with the Ninja backend
+    if os.path.isfile(installfile):
+        with open(installfile, 'rb') as f:
+            installdata = pickle.load(f)
+    else:
+        installdata = None
+
     if options.list_targets:
-        list_targets(coredata, builddata)
+        list_targets(coredata, builddata, installdata)
+    elif options.list_installed:
+        list_installed(installdata)
     elif options.target_files is not None:
         list_target_files(options.target_files, coredata, builddata)
     elif options.buildsystem_files:
@@ -199,6 +243,8 @@ def run(args):
         list_tests(benchmarkdata)
     elif options.dependencies:
         list_deps(coredata)
+    elif options.projectinfo:
+        list_projinfo(builddata)
     else:
         print('No command specified')
         return 1
