@@ -1,6 +1,4 @@
-#!/usr/bin/env python3
-
-# Copyright 2013-2014 The Meson development team
+# Copyright 2013-2016 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,34 +18,36 @@ import sys, struct
 SHT_STRTAB = 3
 DT_NEEDED = 1
 DT_RPATH = 15
+DT_RUNPATH = 29
 DT_STRTAB = 5
 DT_SONAME = 14
+DT_MIPS_RLD_MAP_REL = 1879048245
 
-class DataSizes():
+class DataSizes:
     def __init__(self, ptrsize, is_le):
         if is_le:
             p = '<'
         else:
             p = '>'
-        self.Half = p+'h'
+        self.Half = p + 'h'
         self.HalfSize = 2
-        self.Word = p+'I'
+        self.Word = p + 'I'
         self.WordSize = 4
-        self.Sword = p+'i'
+        self.Sword = p + 'i'
         self.SwordSize = 4
         if ptrsize == 64:
-            self.Addr = p+'Q'
+            self.Addr = p + 'Q'
             self.AddrSize = 8
-            self.Off = p+'Q'
+            self.Off = p + 'Q'
             self.OffSize = 8
-            self.XWord = p+'Q'
+            self.XWord = p + 'Q'
             self.XWordSize = 8
-            self.Sxword = p+'q'
+            self.Sxword = p + 'q'
             self.SxwordSize = 8
         else:
-            self.Addr = p+'I'
+            self.Addr = p + 'I'
             self.AddrSize = 4
-            self.Off = p+'I'
+            self.Off = p + 'I'
             self.OffSize = 4
 
 class DynamicEntry(DataSizes):
@@ -55,8 +55,8 @@ class DynamicEntry(DataSizes):
         super().__init__(ptrsize, is_le)
         self.ptrsize = ptrsize
         if ptrsize == 64:
-            self.d_tag = struct.unpack(self.Sxword, ifile.read(self.SxwordSize))[0];
-            self.val = struct.unpack(self.XWord, ifile.read(self.XWordSize))[0];
+            self.d_tag = struct.unpack(self.Sxword, ifile.read(self.SxwordSize))[0]
+            self.val = struct.unpack(self.XWord, ifile.read(self.XWordSize))[0]
         else:
             self.d_tag = struct.unpack(self.Sword, ifile.read(self.SwordSize))[0]
             self.val = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
@@ -76,29 +76,29 @@ class SectionHeader(DataSizes):
             is_64 = True
         else:
             is_64 = False
-#Elf64_Word
-        self.sh_name = struct.unpack(self.Word, ifile.read(self.WordSize))[0];
-#Elf64_Word
+# Elf64_Word
+        self.sh_name = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
+# Elf64_Word
         self.sh_type = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
-#Elf64_Xword
+# Elf64_Xword
         if is_64:
             self.sh_flags = struct.unpack(self.XWord, ifile.read(self.XWordSize))[0]
         else:
             self.sh_flags = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
-#Elf64_Addr
-        self.sh_addr = struct.unpack(self.Addr, ifile.read(self.AddrSize))[0];
-#Elf64_Off
+# Elf64_Addr
+        self.sh_addr = struct.unpack(self.Addr, ifile.read(self.AddrSize))[0]
+# Elf64_Off
         self.sh_offset = struct.unpack(self.Off, ifile.read(self.OffSize))[0]
-#Elf64_Xword
+# Elf64_Xword
         if is_64:
             self.sh_size = struct.unpack(self.XWord, ifile.read(self.XWordSize))[0]
         else:
             self.sh_size = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
-#Elf64_Word
-        self.sh_link = struct.unpack(self.Word, ifile.read(self.WordSize))[0];
-#Elf64_Word
-        self.sh_info = struct.unpack(self.Word, ifile.read(self.WordSize))[0];
-#Elf64_Xword
+# Elf64_Word
+        self.sh_link = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
+# Elf64_Word
+        self.sh_info = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
+# Elf64_Xword
         if is_64:
             self.sh_addralign = struct.unpack(self.XWord, ifile.read(self.XWordSize))[0]
         else:
@@ -110,37 +110,47 @@ class SectionHeader(DataSizes):
             self.sh_entsize = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
 
 class Elf(DataSizes):
-    def __init__(self, bfile):
+    def __init__(self, bfile, verbose=True):
         self.bfile = bfile
+        self.verbose = verbose
         self.bf = open(bfile, 'r+b')
-        (self.ptrsize, self.is_le) = self.detect_elf_type()
-        super().__init__(self.ptrsize, self.is_le)
-        self.parse_header()
-        self.parse_sections()
-        self.parse_dynamic()
+        try:
+            (self.ptrsize, self.is_le) = self.detect_elf_type()
+            super().__init__(self.ptrsize, self.is_le)
+            self.parse_header()
+            self.parse_sections()
+            self.parse_dynamic()
+        except:
+            self.bf.close()
+            raise
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.bf.close()
 
     def detect_elf_type(self):
         data = self.bf.read(6)
         if data[1:4] != b'ELF':
             # This script gets called to non-elf targets too
             # so just ignore them.
-            print('File "%s" is not an ELF file.' % self.bfile)
+            if self.verbose:
+                print('File "%s" is not an ELF file.' % self.bfile)
             sys.exit(0)
         if data[4] == 1:
             ptrsize = 32
         elif data[4] == 2:
             ptrsize = 64
         else:
-            print('File "%s" has unknown ELF class.' % self.bfile)
-            sys.exit(1)
+            sys.exit('File "%s" has unknown ELF class.' % self.bfile)
         if data[5] == 1:
             is_le = True
         elif data[5] == 2:
             is_le = False
         else:
-            print('File "%s" has unknown ELF endianness.' % self.bfile)
-            sys.exit(1)
-        return (ptrsize, is_le)
+            sys.exit('File "%s" has unknown ELF endianness.' % self.bfile)
+        return ptrsize, is_le
 
     def parse_header(self):
         self.bf.seek(0)
@@ -186,6 +196,8 @@ class Elf(DataSizes):
     def parse_dynamic(self):
         sec = self.find_section(b'.dynamic')
         self.dynamic = []
+        if sec is None:
+            return
         self.bf.seek(sec.sh_offset)
         while True:
             e = DynamicEntry(self.bf, self.ptrsize, self.is_le)
@@ -208,24 +220,35 @@ class Elf(DataSizes):
                 soname = i
             if i.d_tag == DT_STRTAB:
                 strtab = i
+        if soname is None or strtab is None:
+            print("This file does not have a soname")
+            return
         self.bf.seek(strtab.val + soname.val)
         print(self.read_str())
 
-    def get_rpath_offset(self):
+    def get_entry_offset(self, entrynum):
         sec = self.find_section(b'.dynstr')
         for i in self.dynamic:
-            if i.d_tag == DT_RPATH:
+            if i.d_tag == entrynum:
                 return sec.sh_offset + i.val
         return None
 
     def print_rpath(self):
-        offset = self.get_rpath_offset()
+        offset = self.get_entry_offset(DT_RPATH)
         if offset is None:
             print("This file does not have an rpath.")
         else:
             self.bf.seek(offset)
             print(self.read_str())
 
+    def print_runpath(self):
+        offset = self.get_entry_offset(DT_RUNPATH)
+        if offset is None:
+            print("This file does not have a runpath.")
+        else:
+            self.bf.seek(offset)
+            print(self.read_str())
+
     def print_deps(self):
         sec = self.find_section(b'.dynstr')
         deps = []
@@ -250,35 +273,60 @@ class Elf(DataSizes):
             name = self.read_str()
             if name.startswith(prefix):
                 basename = name.split(b'/')[-1]
-                padding = b'\0'*(len(name) - len(basename))
+                padding = b'\0' * (len(name) - len(basename))
                 newname = basename + padding
                 assert(len(newname) == len(name))
                 self.bf.seek(offset)
                 self.bf.write(newname)
 
     def fix_rpath(self, new_rpath):
-        rp_off = self.get_rpath_offset()
+        # The path to search for can be either rpath or runpath.
+        # Fix both of them to be sure.
+        self.fix_rpathtype_entry(new_rpath, DT_RPATH)
+        self.fix_rpathtype_entry(new_rpath, DT_RUNPATH)
+
+    def fix_rpathtype_entry(self, new_rpath, entrynum):
+        if isinstance(new_rpath, str):
+            new_rpath = new_rpath.encode('utf8')
+        rp_off = self.get_entry_offset(entrynum)
         if rp_off is None:
-            print('File does not have rpath. It should be a fully static executable.')
+            if self.verbose:
+                print('File does not have rpath. It should be a fully static executable.')
             return
         self.bf.seek(rp_off)
         old_rpath = self.read_str()
         if len(old_rpath) < len(new_rpath):
-            print("New rpath must not be longer than the old one.")
-        self.bf.seek(rp_off)
-        self.bf.write(new_rpath)
-        self.bf.write(b'\0'*(len(old_rpath) - len(new_rpath) + 1))
+            sys.exit("New rpath must not be longer than the old one.")
+        # The linker does read-only string deduplication. If there is a
+        # string that shares a suffix with the rpath, they might get
+        # dedupped. This means changing the rpath string might break something
+        # completely unrelated. This has already happened once with X.org.
+        # Thus we want to keep this change as small as possible to minimize
+        # the chance of obliterating other strings. It might still happen
+        # but our behaviour is identical to what chrpath does and it has
+        # been in use for ages so based on that this should be rare.
         if len(new_rpath) == 0:
-            self.remove_rpath_entry()
+            self.remove_rpath_entry(entrynum)
+        else:
+            self.bf.seek(rp_off)
+            self.bf.write(new_rpath)
+            self.bf.write(b'\0')
 
-    def remove_rpath_entry(self):
+    def remove_rpath_entry(self, entrynum):
         sec = self.find_section(b'.dynamic')
+        if sec is None:
+            return None
         for (i, entry) in enumerate(self.dynamic):
-            if entry.d_tag == DT_RPATH:
+            if entry.d_tag == entrynum:
                 rpentry = self.dynamic[i]
                 rpentry.d_tag = 0
-                self.dynamic = self.dynamic[:i] + self.dynamic[i+1:] + [rpentry]
-                break;
+                self.dynamic = self.dynamic[:i] + self.dynamic[i + 1:] + [rpentry]
+                break
+        # DT_MIPS_RLD_MAP_REL is relative to the offset of the tag. Adjust it consequently.
+        for entry in self.dynamic[i:]:
+            if entry.d_tag == DT_MIPS_RLD_MAP_REL:
+                entry.val += 2 * (self.ptrsize // 8)
+                break
         self.bf.seek(sec.sh_offset)
         for entry in self.dynamic:
             entry.write(self.bf)
@@ -290,12 +338,13 @@ def run(args):
         print('Don\'t run this unless you know what you are doing.')
         print('%s: <binary file> <prefix>' % sys.argv[0])
         exit(1)
-    e = Elf(args[0])
-    if len(args) == 1:
-        e.print_rpath()
-    else:
-        new_rpath = args[1]
-        e.fix_rpath(new_rpath.encode('utf8'))
+    with Elf(args[0]) as e:
+        if len(args) == 1:
+            e.print_rpath()
+            e.print_runpath()
+        else:
+            new_rpath = args[1]
+            e.fix_rpath(new_rpath)
     return 0
 
 if __name__ == '__main__':
