@@ -21,6 +21,8 @@ import subprocess, sys, os, argparse
 import pickle
 from mesonbuild import build
 from mesonbuild import environment
+from mesonbuild.dependencies import ExternalProgram
+from mesonbuild import mlog
 
 import time, datetime, multiprocessing, json
 import concurrent.futures as conc
@@ -36,6 +38,10 @@ def is_windows():
     platname = platform.system().lower()
     return platname == 'windows' or 'mingw' in platname
 
+def is_cygwin():
+    platname = platform.system().lower()
+    return 'cygwin' in platname
+
 def determine_worker_count():
     varname = 'MESON_TESTTHREADS'
     if varname in os.environ:
@@ -150,7 +156,7 @@ def write_json_log(jsonlogfile, test_name, result):
     jsonlogfile.write(json.dumps(jresult) + '\n')
 
 def run_with_mono(fname):
-    if fname.endswith('.exe') and not is_windows():
+    if fname.endswith('.exe') and not (is_windows() or is_cygwin()):
         return True
     return False
 
@@ -202,7 +208,7 @@ class TestHarness:
 
             child_env.update(test.env)
             if len(test.extra_paths) > 0:
-                child_env['PATH'] += ';'.join([''] + test.extra_paths)
+                child_env['PATH'] += os.pathsep.join([''] + test.extra_paths)
 
             # If MALLOC_PERTURB_ is not set, or if it is set to an empty value,
             # (i.e., the test or the environment don't explicitly set it), set
@@ -278,7 +284,16 @@ class TestHarness:
         result_str = '%s %s  %s%s%s%5.2f s' % \
             (num, name, padding1, result.res, padding2, result.duration)
         if not self.options.quiet or result.res != 'OK':
-            print(result_str)
+            if result.res != 'OK' and mlog.colorize_console:
+                if result.res == 'FAIL' or result.res == 'TIMEOUT':
+                    decorator = mlog.red
+                elif result.res == 'SKIP':
+                    decorator = mlog.yellow
+                else:
+                    sys.exit('Unreachable code was ... well ... reached.')
+                print(decorator(result_str).get_text(True))
+            else:
+                print(result_str)
         result_str += "\n\n" + result.get_log()
         if (result.returncode != GNU_SKIP_RETURNCODE) \
                 and (result.returncode != 0) != result.should_fail:
@@ -570,12 +585,21 @@ def run(args):
         print('Can not be both quiet and verbose at the same time.')
         return 1
 
+    check_bin = None
     if options.gdb:
         options.verbose = True
         if options.wrapper:
             print('Must not specify both a wrapper and gdb at the same time.')
             return 1
+        check_bin = 'gdb'
 
+    if options.wrapper:
+        check_bin = options.wrapper[0]
+
+    if check_bin is not None:
+        exe = ExternalProgram(check_bin, silent=True)
+        if not exe.found():
+            sys.exit("Could not find requested program: %s" % check_bin)
     options.wd = os.path.abspath(options.wd)
 
     if not options.no_rebuild:
