@@ -26,13 +26,16 @@ from mesonbuild import mesonlib
 from mesonbuild import mlog
 from mesonbuild import mesonmain
 from mesonbuild.mesonlib import stringlistify, Popen_safe
+from mesonbuild.coredata import backendlist
 import argparse
 import xml.etree.ElementTree as ET
 import time
 import multiprocessing
 import concurrent.futures as conc
+import re
+
+from run_tests import get_backend_commands, get_backend_args_for_dir, Backend
 
-from mesonbuild.coredata import backendlist
 
 class BuildStep(Enum):
     configure = 1
@@ -40,6 +43,8 @@ class BuildStep(Enum):
     test = 3
     install = 4
     clean = 5
+    validate = 6
+
 
 class TestResult:
     def __init__(self, msg, step, stdo, stde, mlog, conftime=0, buildtime=0, testtime=0):
@@ -52,6 +57,54 @@ class TestResult:
         self.buildtime = buildtime
         self.testtime = testtime
 
+class DummyFuture(conc.Future):
+    '''
+    Dummy Future implementation that executes the provided function when you
+    ask for the result. Used on platforms where sem_open() is not available:
+    MSYS2, OpenBSD, etc: https://bugs.python.org/issue3770
+    '''
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    def set_function(self, fn, *args, **kwargs):
+        self.fn = fn
+        self.fn_args = args
+        self.fn_kwargs = kwargs
+
+    def result(self, **kwargs):
+        try:
+            result = self.fn(*self.fn_args, **self.fn_kwargs)
+        except BaseException as e:
+            self.set_exception(e)
+        else:
+            self.set_result(result)
+        return super().result(**kwargs)
+
+
+class DummyExecutor(conc.Executor):
+    '''
+    Dummy single-thread 'concurrent' executor for use on platforms where
+    sem_open is not available: https://bugs.python.org/issue3770
+    '''
+
+    def __init__(self):
+        from threading import Lock
+        self._shutdown = False
+        self._shutdownLock = Lock()
+
+    def submit(self, fn, *args, **kwargs):
+        with self._shutdownLock:
+            if self._shutdown:
+                raise RuntimeError('Cannot schedule new futures after shutdown')
+            f = DummyFuture()
+            f.set_function(fn, *args, **kwargs)
+            return f
+
+    def shutdown(self, wait=True):
+        with self._shutdownLock:
+            self._shutdown = True
+
+
 class AutoDeletedDir:
     def __init__(self, d):
         self.dir = d
@@ -102,50 +155,36 @@ def stop_handler(signal, frame):
 signal.signal(signal.SIGINT, stop_handler)
 signal.signal(signal.SIGTERM, stop_handler)
 
-# unity_flags = ['--unity']
-unity_flags = []
+# Needed when running cross tests because we don't generate prebuilt files
+compiler = None
 
-backend_flags = None
-compile_commands = None
-test_commands = None
-install_commands = []
-clean_commands = []
-
-def setup_commands(backend):
-    global backend_flags, compile_commands, test_commands, install_commands, clean_commands
+def setup_commands(optbackend):
+    global do_debug, backend, backend_flags
+    global compile_commands, clean_commands, test_commands, install_commands, uninstall_commands
+    backend = optbackend
     msbuild_exe = shutil.which('msbuild')
-    if (backend and backend.startswith('vs')) or (backend is None and msbuild_exe is not None):
-        if backend is None:
-            backend = 'vs2010'
-        backend_flags = ['--backend=' + backend]
-        compile_commands = ['msbuild']
-        test_commands = ['msbuild', 'RUN_TESTS.vcxproj']
-    elif backend == 'xcode' or (backend is None and mesonlib.is_osx()):
-        backend_flags = ['--backend=xcode']
-        compile_commands = ['xcodebuild']
-        test_commands = ['xcodebuild', '-target', 'RUN_TESTS']
-    else:
-        backend_flags = []
-        # We need at least 1.6 because of -w dupbuild=err
-        ninja_command = environment.detect_ninja(version='1.6')
-        if ninja_command is None:
-            raise RuntimeError('Could not find Ninja v1.6 or newer')
-        if do_debug:
-            compile_commands = [ninja_command, '-v']
+    # Auto-detect backend if unspecified
+    if backend is None:
+        if msbuild_exe is not None:
+            backend = 'vs' # Meson will auto-detect VS version to use
+        elif mesonlib.is_osx():
+            backend = 'xcode'
         else:
-            compile_commands = [ninja_command]
-        compile_commands += ['-w', 'dupbuild=err']
-        test_commands = [ninja_command, 'test', 'benchmark']
-        install_commands = [ninja_command, 'install']
-        clean_commands = [ninja_command, 'clean']
-
-def get_compile_commands_for_dir(compile_commands, test_build_dir):
-    if 'msbuild' in compile_commands[0]:
-        sln_name = glob(os.path.join(test_build_dir, '*.sln'))[0]
-        comp = compile_commands + [os.path.split(sln_name)[-1]]
+            backend = 'ninja'
+    # Set backend arguments for Meson
+    if backend.startswith('vs'):
+        backend_flags = ['--backend=' + backend]
+        backend = Backend.vs
+    elif backend == 'xcode':
+        backend_flags = ['--backend=xcode']
+        backend = Backend.xcode
+    elif backend == 'ninja':
+        backend_flags = ['--backend=ninja']
+        backend = Backend.ninja
     else:
-        comp = compile_commands
-    return comp
+        raise RuntimeError('Unknown backend: {!r}'.format(backend))
+    compile_commands, clean_commands, test_commands, install_commands, \
+        uninstall_commands = get_backend_commands(backend, do_debug)
 
 def get_relative_files_list_from_dir(fromdir):
     paths = []
@@ -158,15 +197,21 @@ def get_relative_files_list_from_dir(fromdir):
             paths.append(path)
     return paths
 
-def platform_fix_exe_name(fname):
-    if not fname.endswith('?exe'):
-        return fname
-    fname = fname[:-4]
-    if mesonlib.is_windows():
-        return fname + '.exe'
+def platform_fix_name(fname):
+    if '?lib' in fname:
+        if mesonlib.is_cygwin():
+            fname = re.sub(r'\?lib(.*)\.dll$', r'cyg\1.dll', fname)
+        else:
+            fname = re.sub(r'\?lib', 'lib', fname)
+
+    if fname.endswith('?exe'):
+        fname = fname[:-4]
+        if mesonlib.is_windows() or mesonlib.is_cygwin():
+            return fname + '.exe'
+
     return fname
 
-def validate_install(srcdir, installdir):
+def validate_install(srcdir, installdir, compiler):
     # List of installed files
     info_file = os.path.join(srcdir, 'installed_files.txt')
     # If this exists, the test does not install any other files
@@ -180,13 +225,16 @@ def validate_install(srcdir, installdir):
     elif os.path.exists(info_file):
         with open(info_file) as f:
             for line in f:
-                expected[platform_fix_exe_name(line.strip())] = False
+                expected[platform_fix_name(line.strip())] = False
     # Check if expected files were found
     for fname in expected:
         if os.path.exists(os.path.join(installdir, fname)):
             expected[fname] = True
     for (fname, found) in expected.items():
         if not found:
+            # Ignore missing PDB files if we aren't using cl
+            if fname.endswith('.pdb') and compiler != 'cl':
+                continue
             ret_msg += 'Expected file {0} missing.\n'.format(fname)
     # Check if there are any unexpected files
     found = get_relative_files_list_from_dir(installdir)
@@ -194,7 +242,7 @@ def validate_install(srcdir, installdir):
         # Windows-specific tests check for the existence of installed PDB
         # files, but common tests do not, for obvious reasons. Ignore any
         # extra PDB files found.
-        if fname not in expected and not fname.endswith('.pdb'):
+        if fname not in expected and not fname.endswith('.pdb') and compiler == 'cl':
             ret_msg += 'Extra file {0} found.\n'.format(fname)
     return ret_msg
 
@@ -257,18 +305,18 @@ def parse_test_args(testdir):
         pass
     return args
 
-def run_test(skipped, testdir, extra_args, flags, compile_commands, should_fail):
+def run_test(skipped, testdir, extra_args, compiler, backend, flags, commands, should_fail):
     if skipped:
         return None
     with AutoDeletedDir(tempfile.mkdtemp(prefix='b ', dir='.')) as build_dir:
         with AutoDeletedDir(tempfile.mkdtemp(prefix='i ', dir=os.getcwd())) as install_dir:
             try:
-                return _run_test(testdir, build_dir, install_dir, extra_args, flags, compile_commands, should_fail)
+                return _run_test(testdir, build_dir, install_dir, extra_args, compiler, backend, flags, commands, should_fail)
             finally:
                 mlog.shutdown() # Close the log file because otherwise Windows wets itself.
 
-def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_commands, should_fail):
-    global install_commands, clean_commands
+def _run_test(testdir, test_build_dir, install_dir, extra_args, compiler, backend, flags, commands, should_fail):
+    compile_commands, clean_commands, install_commands, uninstall_commands = commands
     test_args = parse_test_args(testdir)
     gen_start = time.time()
     # Configure in-process
@@ -289,9 +337,9 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
     if returncode != 0:
         return TestResult('Generating the build system failed.', BuildStep.configure, stdo, stde, mesonlog, gen_time)
     # Build with subprocess
-    comp = get_compile_commands_for_dir(compile_commands, test_build_dir)
+    dir_args = get_backend_args_for_dir(backend, test_build_dir)
     build_start = time.time()
-    pc, o, e = Popen_safe(comp, cwd=test_build_dir)
+    pc, o, e = Popen_safe(compile_commands + dir_args, cwd=test_build_dir)
     build_time = time.time() - build_start
     stdo += o
     stde += e
@@ -318,25 +366,26 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
         return TestResult('Test that should have failed to run unit tests succeeded', BuildStep.test, stdo, stde, mesonlog, gen_time)
     if returncode != 0:
         return TestResult('Running unit tests failed.', BuildStep.test, stdo, stde, mesonlog, gen_time, build_time, test_time)
-    if len(install_commands) == 0:
-        return TestResult('', BuildStep.install, '', '', mesonlog, gen_time, build_time, test_time)
-    env = os.environ.copy()
-    env['DESTDIR'] = install_dir
-    # Install with subprocess
-    pi, o, e = Popen_safe(install_commands, cwd=test_build_dir, env=env)
-    stdo += o
-    stde += e
-    if pi.returncode != 0:
-        return TestResult('Running install failed.', BuildStep.install, stdo, stde, mesonlog, gen_time, build_time, test_time)
-    if len(clean_commands) != 0:
+    # Do installation, if the backend supports it
+    if len(install_commands) != 0:
         env = os.environ.copy()
-        # Clean with subprocess
-        pi, o, e = Popen_safe(clean_commands, cwd=test_build_dir, env=env)
+        env['DESTDIR'] = install_dir
+        # Install with subprocess
+        pi, o, e = Popen_safe(install_commands, cwd=test_build_dir, env=env)
         stdo += o
         stde += e
         if pi.returncode != 0:
-            return TestResult('Running clean failed.', BuildStep.clean, stdo, stde, mesonlog, gen_time, build_time, test_time)
-    return TestResult(validate_install(testdir, install_dir), BuildStep.clean, stdo, stde, mesonlog, gen_time, build_time, test_time)
+            return TestResult('Running install failed.', BuildStep.install, stdo, stde, mesonlog, gen_time, build_time, test_time)
+    # Clean with subprocess
+    env = os.environ.copy()
+    pi, o, e = Popen_safe(clean_commands + dir_args, cwd=test_build_dir, env=env)
+    stdo += o
+    stde += e
+    if pi.returncode != 0:
+        return TestResult('Running clean failed.', BuildStep.clean, stdo, stde, mesonlog, gen_time, build_time, test_time)
+    if len(install_commands) == 0:
+        return TestResult('', BuildStep.install, '', '', mesonlog, gen_time, build_time, test_time)
+    return TestResult(validate_install(testdir, install_dir, compiler), BuildStep.validate, stdo, stde, mesonlog, gen_time, build_time, test_time)
 
 def gather_tests(testdir):
     tests = [t.replace('\\', '/').split('/', 2)[2] for t in glob(os.path.join(testdir, '*'))]
@@ -361,23 +410,6 @@ def have_java():
         return True
     return False
 
-def using_backend(backends):
-    if isinstance(backends, str):
-        backends = (backends,)
-    for backend in backends:
-        if backend == 'ninja':
-            if not backend_flags:
-                return True
-        elif backend == 'xcode':
-            if backend_flags == '--backend=xcode':
-                return True
-        elif backend == 'vs':
-            if backend_flags.startswith('--backend=vs'):
-                return True
-        else:
-            raise AssertionError('Unknown backend type: ' + backend)
-    return False
-
 def detect_tests_to_run():
     all_tests = []
     all_tests.append(('common', gather_tests('test cases/common'), False))
@@ -387,18 +419,18 @@ def detect_tests_to_run():
     all_tests.append(('prebuilt', gather_tests('test cases/prebuilt'), False))
 
     all_tests.append(('platform-osx', gather_tests('test cases/osx'), False if mesonlib.is_osx() else True))
-    all_tests.append(('platform-windows', gather_tests('test cases/windows'), False if mesonlib.is_windows() else True))
+    all_tests.append(('platform-windows', gather_tests('test cases/windows'), False if mesonlib.is_windows() or mesonlib.is_cygwin() else True))
     all_tests.append(('platform-linux', gather_tests('test cases/linuxlike'), False if not (mesonlib.is_osx() or mesonlib.is_windows()) else True))
-    all_tests.append(('framework', gather_tests('test cases/frameworks'), False if not mesonlib.is_osx() and not mesonlib.is_windows() else True))
-    all_tests.append(('java', gather_tests('test cases/java'), False if using_backend('ninja') and not mesonlib.is_osx() and have_java() else True))
-    all_tests.append(('C#', gather_tests('test cases/csharp'), False if using_backend('ninja') and shutil.which('mcs') else True))
-    all_tests.append(('vala', gather_tests('test cases/vala'), False if using_backend('ninja') and shutil.which('valac') else True))
-    all_tests.append(('rust', gather_tests('test cases/rust'), False if using_backend('ninja') and shutil.which('rustc') else True))
-    all_tests.append(('d', gather_tests('test cases/d'), False if using_backend('ninja') and have_d_compiler() else True))
-    all_tests.append(('objective c', gather_tests('test cases/objc'), False if using_backend(('ninja', 'xcode')) and not mesonlib.is_windows() else True))
-    all_tests.append(('fortran', gather_tests('test cases/fortran'), False if using_backend('ninja') and shutil.which('gfortran') else True))
-    all_tests.append(('swift', gather_tests('test cases/swift'), False if using_backend(('ninja', 'xcode')) and shutil.which('swiftc') else True))
-    all_tests.append(('python3', gather_tests('test cases/python3'), False if using_backend('ninja') and shutil.which('python3') else True))
+    all_tests.append(('framework', gather_tests('test cases/frameworks'), False if not mesonlib.is_osx() and not mesonlib.is_windows() and not mesonlib.is_cygwin() else True))
+    all_tests.append(('java', gather_tests('test cases/java'), False if backend is Backend.ninja and not mesonlib.is_osx() and have_java() else True))
+    all_tests.append(('C#', gather_tests('test cases/csharp'), False if backend is Backend.ninja and shutil.which('mcs') else True))
+    all_tests.append(('vala', gather_tests('test cases/vala'), False if backend is Backend.ninja and shutil.which('valac') else True))
+    all_tests.append(('rust', gather_tests('test cases/rust'), False if backend is Backend.ninja and shutil.which('rustc') else True))
+    all_tests.append(('d', gather_tests('test cases/d'), False if backend is Backend.ninja and have_d_compiler() else True))
+    all_tests.append(('objective c', gather_tests('test cases/objc'), False if backend in (Backend.ninja, Backend.xcode) and not mesonlib.is_windows() else True))
+    all_tests.append(('fortran', gather_tests('test cases/fortran'), False if backend is Backend.ninja and shutil.which('gfortran') else True))
+    all_tests.append(('swift', gather_tests('test cases/swift'), False if backend in (Backend.ninja, Backend.xcode) and shutil.which('swiftc') else True))
+    all_tests.append(('python3', gather_tests('test cases/python3'), False if backend is Backend.ninja and shutil.which('python3') else True))
     return all_tests
 
 def run_tests(all_tests, log_name_base, extra_args):
@@ -413,6 +445,7 @@ def run_tests(all_tests, log_name_base, extra_args):
     passing_tests = 0
     failing_tests = 0
     skipped_tests = 0
+    commands = (compile_commands, clean_commands, install_commands, uninstall_commands)
 
     try:
         # This fails in some CI environments for unknown reasons.
@@ -421,7 +454,11 @@ def run_tests(all_tests, log_name_base, extra_args):
         print('Could not determine number of CPUs due to the following reason:' + str(e))
         print('Defaulting to using only one process')
         num_workers = 1
-    executor = conc.ProcessPoolExecutor(max_workers=num_workers)
+    try:
+        executor = conc.ProcessPoolExecutor(max_workers=num_workers)
+    except ImportError:
+        print('Platform doesn\'t ProcessPoolExecutor, falling back to single-threaded testing\n')
+        executor = DummyExecutor()
 
     for name, test_cases, skipped in all_tests:
         current_suite = ET.SubElement(junit_root, 'testsuite', {'name': name, 'tests': str(len(test_cases))})
@@ -438,9 +475,10 @@ def run_tests(all_tests, log_name_base, extra_args):
             should_fail = False
             if name.startswith('failing'):
                 should_fail = name.split('failing-')[1]
-            result = executor.submit(run_test, skipped, t, extra_args, unity_flags + backend_flags, compile_commands, should_fail)
+            result = executor.submit(run_test, skipped, t, extra_args, compiler, backend, backend_flags, commands, should_fail)
             futures.append((testname, t, result))
         for (testname, t, result) in futures:
+            sys.stdout.flush()
             result = result.result()
             if result is None or 'MESON_SKIP_TEST' in result.stdo:
                 print('Skipping:', t)
@@ -532,6 +570,7 @@ def generate_pb_static(compiler, object_suffix, static_suffix):
     return stlibfile
 
 def generate_prebuilt():
+    global compiler
     static_suffix = 'a'
     if shutil.which('cl'):
         compiler = 'cl'
@@ -551,7 +590,7 @@ def generate_prebuilt():
     return objectfile, stlibfile
 
 def check_meson_commands_work():
-    global meson_command, compile_commands, test_commands, install_commands
+    global backend, meson_command, compile_commands, test_commands, install_commands
     testdir = 'test cases/common/1 trivial'
     with AutoDeletedDir(tempfile.mkdtemp(prefix='b ', dir='.')) as build_dir:
         print('Checking that configuring works...')
@@ -560,8 +599,8 @@ def check_meson_commands_work():
         if pc.returncode != 0:
             raise RuntimeError('Failed to configure {!r}:\n{}\n{}'.format(testdir, e, o))
         print('Checking that building works...')
-        compile_cmd = get_compile_commands_for_dir(compile_commands, build_dir)
-        pc, o, e = Popen_safe(compile_cmd, cwd=build_dir)
+        dir_args = get_backend_args_for_dir(backend, build_dir)
+        pc, o, e = Popen_safe(compile_commands + dir_args, cwd=build_dir)
         if pc.returncode != 0:
             raise RuntimeError('Failed to build {!r}:\n{}\n{}'.format(testdir, e, o))
         print('Checking that testing works...')
@@ -583,21 +622,6 @@ if __name__ == '__main__':
     options = parser.parse_args()
     setup_commands(options.backend)
 
-    # Appveyor sets the `platform` environment variable which completely messes
-    # up building with the vs2010 and vs2015 backends.
-    #
-    # Specifically, MSBuild reads the `platform` environment variable to set
-    # the configured value for the platform (Win32/x64/arm), which breaks x86
-    # builds.
-    #
-    # Appveyor setting this also breaks our 'native build arch' detection for
-    # Windows in environment.py:detect_windows_arch() by overwriting the value
-    # of `platform` set by vcvarsall.bat.
-    #
-    # While building for x86, `platform` should be unset.
-    if 'APPVEYOR' in os.environ and os.environ['arch'] == 'x86':
-        os.environ.pop('platform')
-
     script_dir = os.path.split(__file__)[0]
     if script_dir != '':
         os.chdir(script_dir)
