@@ -24,14 +24,13 @@ from pathlib import PurePath
 import mesonbuild.compilers
 import mesonbuild.environment
 import mesonbuild.mesonlib
-from mesonbuild.mesonlib import is_windows, is_osx
-from mesonbuild.environment import detect_ninja, Environment
+from mesonbuild.mesonlib import is_windows, is_osx, is_cygwin
+from mesonbuild.environment import Environment
 from mesonbuild.dependencies import PkgConfigDependency, ExternalProgram
 
-if is_windows():
-    exe_suffix = '.exe'
-else:
-    exe_suffix = ''
+from run_tests import exe_suffix, get_fake_options, FakeEnvironment
+from run_tests import get_builddir_target_args, get_backend_commands, Backend
+
 
 def get_soname(fname):
     # HACK, fix to not use shell.
@@ -44,20 +43,6 @@ def get_soname(fname):
             return m.group(1)
     raise RuntimeError('Could not determine soname:\n\n' + raw_out)
 
-def get_fake_options(prefix):
-    import argparse
-    opts = argparse.Namespace()
-    opts.cross_file = None
-    opts.prefix = prefix
-    return opts
-
-class FakeEnvironment(object):
-    def __init__(self):
-        self.cross_info = None
-
-    def is_cross_build(self):
-        return False
-
 class InternalTests(unittest.TestCase):
 
     def test_version_number(self):
@@ -345,16 +330,37 @@ class BasePlatformTests(unittest.TestCase):
         self.prefix = '/usr'
         self.libdir = os.path.join(self.prefix, 'lib')
         self.installdir = os.path.join(self.builddir, 'install')
-        self.meson_command = [sys.executable, os.path.join(src_root, 'meson.py')]
+        # Get the backend
+        # FIXME: Extract this from argv?
+        self.backend = getattr(Backend, os.environ.get('MESON_UNIT_TEST_BACKEND', 'ninja'))
+        self.meson_command = [sys.executable, os.path.join(src_root, 'meson.py'),
+                              '--backend=' + self.backend.name]
         self.mconf_command = [sys.executable, os.path.join(src_root, 'mesonconf.py')]
         self.mintro_command = [sys.executable, os.path.join(src_root, 'mesonintrospect.py')]
         self.mtest_command = [sys.executable, os.path.join(src_root, 'mesontest.py'), '-C', self.builddir]
-        self.ninja_command = [detect_ninja(), '-C', self.builddir]
+        # Backend-specific build commands
+        self.build_command, self.clean_command, self.test_command, self.install_command, \
+            self.uninstall_command = get_backend_commands(self.backend)
+        # Test directories
         self.common_test_dir = os.path.join(src_root, 'test cases/common')
         self.vala_test_dir = os.path.join(src_root, 'test cases/vala')
         self.framework_test_dir = os.path.join(src_root, 'test cases/frameworks')
         self.unit_test_dir = os.path.join(src_root, 'test cases/unit')
+        # Misc stuff
         self.orig_env = os.environ.copy()
+        if self.backend is Backend.ninja:
+            self.no_rebuild_stdout = 'ninja: no work to do.'
+        else:
+            # VS doesn't have a stable output when no changes are done
+            # XCode backend is untested with unit tests, help welcome!
+            self.no_rebuild_stdout = 'UNKNOWN BACKEND {!r}'.format(self.backend.name)
+
+    def ensure_backend_detects_changes(self):
+        # This is needed to increase the difference between build.ninja's
+        # timestamp and the timestamp of whatever you changed due to a Ninja
+        # bug: https://github.com/ninja-build/ninja/issues/371
+        if self.backend is Backend.ninja:
+            time.sleep(1)
 
     def _print_meson_log(self):
         log = os.path.join(self.logdir, 'meson-log.txt')
@@ -369,14 +375,14 @@ class BasePlatformTests(unittest.TestCase):
         os.environ = self.orig_env
         super().tearDown()
 
-    def _run(self, command):
+    def _run(self, command, workdir=None):
         '''
         Run a command while printing the stdout and stderr to stdout,
         and also return a copy of it
         '''
         p = subprocess.Popen(command, stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT, env=os.environ.copy(),
-                             universal_newlines=True)
+                             universal_newlines=True, cwd=workdir)
         output = p.communicate()[0]
         print(output)
         if p.returncode != 0:
@@ -384,8 +390,11 @@ class BasePlatformTests(unittest.TestCase):
         return output
 
     def init(self, srcdir, extra_args=None, default_args=True):
+        self.assertTrue(os.path.exists(srcdir))
         if extra_args is None:
             extra_args = []
+        if not isinstance(extra_args, list):
+            extra_args = [extra_args]
         args = [srcdir, self.builddir]
         if default_args:
             args += ['--prefix', self.prefix,
@@ -397,42 +406,67 @@ class BasePlatformTests(unittest.TestCase):
             raise
         self.privatedir = os.path.join(self.builddir, 'meson-private')
 
-    def build(self, extra_args=None):
+    def build(self, target=None, extra_args=None):
         if extra_args is None:
             extra_args = []
-        self._run(self.ninja_command + extra_args)
+        # Add arguments for building the target (if specified),
+        # and using the build dir (if required, with VS)
+        args = get_builddir_target_args(self.backend, self.builddir, target)
+        return self._run(self.build_command + args + extra_args, workdir=self.builddir)
+
+    def clean(self):
+        dir_args = get_builddir_target_args(self.backend, self.builddir, None)
+        self._run(self.clean_command + dir_args, workdir=self.builddir)
 
     def run_tests(self):
-        self._run(self.ninja_command + ['test'])
+        self._run(self.test_command, workdir=self.builddir)
 
     def install(self):
+        if self.backend is not Backend.ninja:
+            raise unittest.SkipTest('{!r} backend can\'t install files'.format(self.backend.name))
         os.environ['DESTDIR'] = self.installdir
-        self._run(self.ninja_command + ['install'])
+        self._run(self.install_command, workdir=self.builddir)
 
     def uninstall(self):
-        self._run(self.ninja_command + ['uninstall'])
+        self._run(self.uninstall_command, workdir=self.builddir)
 
     def run_target(self, target):
         '''
         Run a Ninja target while printing the stdout and stderr to stdout,
         and also return a copy of it
         '''
-        return self._run(self.ninja_command + [target])
+        return self.build(target=target)
 
     def setconf(self, arg, will_build=True):
-        # This is needed to increase the difference between build.ninja's
-        # timestamp and coredata.dat's timestamp due to a Ninja bug.
-        # https://github.com/ninja-build/ninja/issues/371
         if will_build:
-            time.sleep(1)
+            self.ensure_backend_detects_changes()
         self._run(self.mconf_command + [arg, self.builddir])
 
     def wipe(self):
         shutil.rmtree(self.builddir)
 
+    def utime(self, f):
+        self.ensure_backend_detects_changes()
+        os.utime(f)
+
     def get_compdb(self):
+        if self.backend is not Backend.ninja:
+            raise unittest.SkipTest('Compiler db not available with {} backend'.format(self.backend.name))
         with open(os.path.join(self.builddir, 'compile_commands.json')) as ifile:
-            return json.load(ifile)
+            contents = json.load(ifile)
+        # If Ninja is using .rsp files, generate them, read their contents, and
+        # replace it as the command for all compile commands in the parsed json.
+        if len(contents) > 0 and contents[0]['command'].endswith('.rsp'):
+            # Pretend to build so that the rsp files are generated
+            self.build(extra_args=['-d', 'keeprsp', '-n'])
+            for each in contents:
+                # Extract the actual command from the rsp file
+                compiler, rsp = each['command'].split(' @')
+                rsp = os.path.join(self.builddir, rsp)
+                # Replace the command with its contents
+                with open(rsp, 'r', encoding='utf-8') as f:
+                    each['command'] = compiler + ' ' + f.read()
+        return contents
 
     def get_meson_log(self):
         with open(os.path.join(self.builddir, 'meson-logs', 'meson-log.txt')) as f:
@@ -468,6 +502,40 @@ class BasePlatformTests(unittest.TestCase):
         path_basename = PurePath(path).parts[-1]
         self.assertEqual(PurePath(path_basename), PurePath(basename), msg)
 
+    def assertBuildIsNoop(self):
+        ret = self.build()
+        if self.backend is Backend.ninja:
+            self.assertEqual(ret.split('\n')[-2], self.no_rebuild_stdout)
+        elif self.backend is Backend.vs:
+            # Ensure that some target said that no rebuild was done
+            self.assertIn('CustomBuild:\n  All outputs are up-to-date.', ret)
+            self.assertIn('ClCompile:\n  All outputs are up-to-date.', ret)
+            self.assertIn('Link:\n  All outputs are up-to-date.', ret)
+            # Ensure that no targets were built
+            clre = re.compile('ClCompile:\n [^\n]*cl', flags=re.IGNORECASE)
+            linkre = re.compile('Link:\n [^\n]*link', flags=re.IGNORECASE)
+            self.assertNotRegex(ret, clre)
+            self.assertNotRegex(ret, linkre)
+        elif self.backend is Backend.xcode:
+            raise unittest.SkipTest('Please help us fix this test on the xcode backend')
+        else:
+            raise RuntimeError('Invalid backend: {!r}'.format(self.backend.name))
+
+    def assertRebuiltTarget(self, target):
+        ret = self.build()
+        if self.backend is Backend.ninja:
+            self.assertIn('Linking target {}'.format(target), ret)
+        elif self.backend is Backend.vs:
+            # Ensure that this target was rebuilt
+            clre = re.compile('ClCompile:\n [^\n]*cl[^\n]*' + target, flags=re.IGNORECASE)
+            linkre = re.compile('Link:\n [^\n]*link[^\n]*' + target, flags=re.IGNORECASE)
+            self.assertRegex(ret, clre)
+            self.assertRegex(ret, linkre)
+        elif self.backend is Backend.xcode:
+            raise unittest.SkipTest('Please help us fix this test on the xcode backend')
+        else:
+            raise RuntimeError('Invalid backend: {!r}'.format(self.backend.name))
+
 
 class AllPlatformTests(BasePlatformTests):
     '''
@@ -540,8 +608,10 @@ class AllPlatformTests(BasePlatformTests):
                           get_fake_options(self.prefix), [])
         cc = env.detect_c_compiler(False)
         static_linker = env.detect_static_linker(cc)
+        if is_windows():
+            raise unittest.SkipTest('https://github.com/mesonbuild/meson/issues/1526')
         if not isinstance(static_linker, mesonbuild.compilers.ArLinker):
-                raise unittest.SkipTest('static linker is not `ar`')
+            raise unittest.SkipTest('static linker is not `ar`')
         # Configure
         self.init(testdir)
         # Get name of static library
@@ -597,6 +667,8 @@ class AllPlatformTests(BasePlatformTests):
         Tests that the Meson introspection API exposes install filenames correctly
         https://github.com/mesonbuild/meson/issues/829
         '''
+        if self.backend is not Backend.ninja:
+            raise unittest.SkipTest('{!r} backend can\'t install files'.format(self.backend.name))
         testdir = os.path.join(self.common_test_dir, '8 install')
         self.init(testdir)
         intro = self.introspect('--targets')
@@ -706,26 +778,24 @@ class AllPlatformTests(BasePlatformTests):
         exe = os.path.join(self.builddir, 'fooprog' + exe_suffix)
         self.assertTrue(os.path.exists(genfile))
         self.assertFalse(os.path.exists(exe))
-        self._run(self.ninja_command + ['fooprog' + exe_suffix])
+        self.build(target=('fooprog' + exe_suffix))
         self.assertTrue(os.path.exists(exe))
 
     def test_internal_include_order(self):
         testdir = os.path.join(self.common_test_dir, '138 include order')
         self.init(testdir)
+        execmd = fxecmd = None
         for cmd in self.get_compdb():
-            if cmd['file'].endswith('/main.c'):
-                cmd = cmd['command']
-                break
-        else:
-            raise Exception('Could not find main.c command')
-        if cmd.endswith('.rsp'):
-            # Pretend to build so that the rsp files are generated
-            self.build(['-d', 'keeprsp', '-n'])
-            # Extract the actual command from the rsp file
-            rsp = os.path.join(self.builddir, cmd.split('cl @')[1])
-            with open(rsp, 'r', encoding='utf-8') as f:
-                cmd = f.read()
-        incs = [a for a in shlex.split(cmd) if a.startswith("-I")]
+            if 'someexe' in cmd['command']:
+                execmd = cmd['command']
+                continue
+            if 'somefxe' in cmd['command']:
+                fxecmd = cmd['command']
+                continue
+        if not execmd or not fxecmd:
+            raise Exception('Could not find someexe and somfxe commands')
+        # Check include order for 'someexe'
+        incs = [a for a in shlex.split(execmd) if a.startswith("-I")]
         self.assertEqual(len(incs), 8)
         # target private dir
         self.assertPathEqual(incs[0], "-Isub4/someexe@exe")
@@ -743,6 +813,27 @@ class AllPlatformTests(BasePlatformTests):
         self.assertPathEqual(incs[6], "-Isub1")
         # target internal dependency include_directories: source dir
         self.assertPathBasenameEqual(incs[7], 'sub1')
+        # Check include order for 'somefxe'
+        incs = [a for a in shlex.split(fxecmd) if a.startswith('-I')]
+        self.assertEqual(len(incs), 9)
+        # target private dir
+        self.assertPathEqual(incs[0], '-Isomefxe@exe')
+        # target build dir
+        self.assertPathEqual(incs[1], '-I.')
+        # target source dir
+        self.assertPathBasenameEqual(incs[2], os.path.basename(testdir))
+        # target internal dependency correct include_directories: build dir
+        self.assertPathEqual(incs[3], "-Isub4")
+        # target internal dependency correct include_directories: source dir
+        self.assertPathBasenameEqual(incs[4], 'sub4')
+        # target internal dependency dep include_directories: build dir
+        self.assertPathEqual(incs[5], "-Isub1")
+        # target internal dependency dep include_directories: source dir
+        self.assertPathBasenameEqual(incs[6], 'sub1')
+        # target internal dependency wrong include_directories: build dir
+        self.assertPathEqual(incs[7], "-Isub2")
+        # target internal dependency wrong include_directories: source dir
+        self.assertPathBasenameEqual(incs[8], 'sub2')
 
     def test_compiler_detection(self):
         '''
@@ -798,6 +889,8 @@ class AllPlatformTests(BasePlatformTests):
                     self.assertEqual(cc.gcc_type, mesonbuild.compilers.GCC_OSX)
                 elif is_windows():
                     self.assertEqual(cc.gcc_type, mesonbuild.compilers.GCC_MINGW)
+                elif is_cygwin():
+                    self.assertEqual(cc.gcc_type, mesonbuild.compilers.GCC_CYGWIN)
                 else:
                     self.assertEqual(cc.gcc_type, mesonbuild.compilers.GCC_STANDARD)
             if isinstance(cc, clang):
@@ -845,6 +938,139 @@ class AllPlatformTests(BasePlatformTests):
             self.assertEqual(wcc.get_exelist(), wrappercc)
             self.assertEqual(wlinker.get_exelist(), wrapperlinker)
 
+    def test_always_prefer_c_compiler_for_asm(self):
+        testdir = os.path.join(self.common_test_dir, '141 c cpp and asm')
+        # Skip if building with MSVC
+        env = Environment(testdir, self.builddir, self.meson_command,
+                          get_fake_options(self.prefix), [])
+        if env.detect_c_compiler(False).get_id() == 'msvc':
+            raise unittest.SkipTest('MSVC can\'t compile assembly')
+        self.init(testdir)
+        commands = {'c-asm': {}, 'cpp-asm': {}, 'cpp-c-asm': {}, 'c-cpp-asm': {}}
+        for cmd in self.get_compdb():
+            # Get compiler
+            split = shlex.split(cmd['command'])
+            if split[0] == 'ccache':
+                compiler = split[1]
+            else:
+                compiler = split[0]
+            # Classify commands
+            if 'Ic-asm' in cmd['command']:
+                if cmd['file'].endswith('.S'):
+                    commands['c-asm']['asm'] = compiler
+                elif cmd['file'].endswith('.c'):
+                    commands['c-asm']['c'] = compiler
+                else:
+                    raise AssertionError('{!r} found in cpp-asm?'.format(cmd['command']))
+            elif 'Icpp-asm' in cmd['command']:
+                if cmd['file'].endswith('.S'):
+                    commands['cpp-asm']['asm'] = compiler
+                elif cmd['file'].endswith('.cpp'):
+                    commands['cpp-asm']['cpp'] = compiler
+                else:
+                    raise AssertionError('{!r} found in cpp-asm?'.format(cmd['command']))
+            elif 'Ic-cpp-asm' in cmd['command']:
+                if cmd['file'].endswith('.S'):
+                    commands['c-cpp-asm']['asm'] = compiler
+                elif cmd['file'].endswith('.c'):
+                    commands['c-cpp-asm']['c'] = compiler
+                elif cmd['file'].endswith('.cpp'):
+                    commands['c-cpp-asm']['cpp'] = compiler
+                else:
+                    raise AssertionError('{!r} found in c-cpp-asm?'.format(cmd['command']))
+            elif 'Icpp-c-asm' in cmd['command']:
+                if cmd['file'].endswith('.S'):
+                    commands['cpp-c-asm']['asm'] = compiler
+                elif cmd['file'].endswith('.c'):
+                    commands['cpp-c-asm']['c'] = compiler
+                elif cmd['file'].endswith('.cpp'):
+                    commands['cpp-c-asm']['cpp'] = compiler
+                else:
+                    raise AssertionError('{!r} found in cpp-c-asm?'.format(cmd['command']))
+            else:
+                raise AssertionError('Unknown command {!r} found'.format(cmd['command']))
+        # Check that .S files are always built with the C compiler
+        self.assertEqual(commands['c-asm']['asm'], commands['c-asm']['c'])
+        self.assertEqual(commands['c-asm']['asm'], commands['cpp-asm']['asm'])
+        self.assertEqual(commands['cpp-asm']['asm'], commands['c-cpp-asm']['c'])
+        self.assertEqual(commands['c-cpp-asm']['asm'], commands['c-cpp-asm']['c'])
+        self.assertEqual(commands['cpp-c-asm']['asm'], commands['cpp-c-asm']['c'])
+        self.assertNotEqual(commands['cpp-asm']['asm'], commands['cpp-asm']['cpp'])
+        self.assertNotEqual(commands['c-cpp-asm']['c'], commands['c-cpp-asm']['cpp'])
+        self.assertNotEqual(commands['cpp-c-asm']['c'], commands['cpp-c-asm']['cpp'])
+        # Check that the c-asm target is always linked with the C linker
+        build_ninja = os.path.join(self.builddir, 'build.ninja')
+        with open(build_ninja, 'r', encoding='utf-8') as f:
+            contents = f.read()
+            m = re.search('build c-asm.*: c_LINKER', contents)
+        self.assertIsNotNone(m, msg=contents)
+
+    def test_preprocessor_checks_CPPFLAGS(self):
+        '''
+        Test that preprocessor compiler checks read CPPFLAGS but not CFLAGS
+        '''
+        testdir = os.path.join(self.common_test_dir, '140 get define')
+        define = 'MESON_TEST_DEFINE_VALUE'
+        # NOTE: this list can't have \n, ' or "
+        # \n is never substituted by the GNU pre-processor via a -D define
+        # ' and " confuse shlex.split() even when they are escaped
+        # % and # confuse the MSVC preprocessor
+        value = 'spaces and fun!@$^&*()-=_+{}[]:;<>?,./~`'
+        os.environ['CPPFLAGS'] = '-D{}="{}"'.format(define, value)
+        os.environ['CFLAGS'] = '-DMESON_FAIL_VALUE=cflags-read'.format(define)
+        self.init(testdir, ['-D{}={}'.format(define, value)])
+
+    def test_custom_target_exe_data_deterministic(self):
+        testdir = os.path.join(self.common_test_dir, '117 custom target capture')
+        self.init(testdir)
+        meson_exe_dat1 = glob(os.path.join(self.privatedir, 'meson_exe*.dat'))
+        self.wipe()
+        self.init(testdir)
+        meson_exe_dat2 = glob(os.path.join(self.privatedir, 'meson_exe*.dat'))
+        self.assertListEqual(meson_exe_dat1, meson_exe_dat2)
+
+    def test_source_changes_cause_rebuild(self):
+        '''
+        Test that changes to sources and headers cause rebuilds, but not
+        changes to unused files (as determined by the dependency file) in the
+        input files list.
+        '''
+        testdir = os.path.join(self.common_test_dir, '22 header in file list')
+        self.init(testdir)
+        self.build()
+        # Immediately rebuilding should not do anything
+        self.assertBuildIsNoop()
+        # Changing mtime of header.h should rebuild everything
+        self.utime(os.path.join(testdir, 'header.h'))
+        self.assertRebuiltTarget('prog')
+
+    def test_custom_target_changes_cause_rebuild(self):
+        '''
+        Test that in a custom target, changes to the input files, the
+        ExternalProgram, and any File objects on the command-line cause
+        a rebuild.
+        '''
+        testdir = os.path.join(self.common_test_dir, '64 custom header generator')
+        self.init(testdir)
+        self.build()
+        # Immediately rebuilding should not do anything
+        self.assertBuildIsNoop()
+        # Changing mtime of these should rebuild everything
+        for f in ('input.def', 'makeheader.py', 'somefile.txt'):
+            self.utime(os.path.join(testdir, f))
+            self.assertRebuiltTarget('prog')
+
+    def test_static_library_lto(self):
+        '''
+        Test that static libraries can be built with LTO and linked to
+        executables. On Linux, this requires the use of gcc-ar.
+        https://github.com/mesonbuild/meson/issues/1646
+        '''
+        testdir = os.path.join(self.common_test_dir, '5 linkstatic')
+        self.init(testdir, extra_args='-Db_lto=true')
+        self.build()
+        self.run_tests()
+
 
 class WindowsTests(BasePlatformTests):
     '''
@@ -990,6 +1216,25 @@ class LinuxlikeTests(BasePlatformTests):
         self.assertIn("'-Werror'", vala_command)
         self.assertIn("'-Werror'", c_command)
 
+    def test_qt5dependency_pkgconfig_detection(self):
+        '''
+        Test that qt4 and qt5 detection with pkgconfig works.
+        '''
+        # Verify Qt4 or Qt5 can be found with pkg-config
+        if not shutil.which('pkg-config'):
+            raise unittest.SkipTest('pkg-config not found')
+        qt4 = subprocess.call(['pkg-config', '--exists', 'QtCore'])
+        qt5 = subprocess.call(['pkg-config', '--exists', 'Qt5Core'])
+        if qt4 != 0 or qt5 != 0:
+            raise unittest.SkipTest('Qt not found with pkg-config')
+        testdir = os.path.join(self.framework_test_dir, '4 qt')
+        self.init(testdir, ['-Dmethod=pkg-config'])
+        # Confirm that the dependency was found with qmake
+        msg = 'Qt4 native `pkg-config` dependency (modules: Core, Gui) found: YES\n'
+        msg2 = 'Qt5 native `pkg-config` dependency (modules: Core, Gui) found: YES\n'
+        mesonlog = self.get_meson_log()
+        self.assertTrue(msg in mesonlog or msg2 in mesonlog)
+
     def test_qt5dependency_qmake_detection(self):
         '''
         Test that qt5 detection with qmake works. This can't be an ordinary
@@ -1006,10 +1251,8 @@ class LinuxlikeTests(BasePlatformTests):
             if 'Qt version 5' not in output and 'qt5' not in output:
                 raise unittest.SkipTest('Qmake found, but it is not for Qt 5.')
         # Disable pkg-config codepath and force searching with qmake/qmake-qt5
-        os.environ['PKG_CONFIG_LIBDIR'] = self.builddir
-        os.environ['PKG_CONFIG_PATH'] = self.builddir
         testdir = os.path.join(self.framework_test_dir, '4 qt')
-        self.init(testdir)
+        self.init(testdir, ['-Dmethod=qmake'])
         # Confirm that the dependency was found with qmake
         msg = 'Qt5 native `qmake-qt5` dependency (modules: Core) found: YES\n'
         msg2 = 'Qt5 native `qmake` dependency (modules: Core) found: YES\n'
@@ -1093,15 +1336,6 @@ class LinuxlikeTests(BasePlatformTests):
             Oargs = [arg for arg in cmd if arg.startswith('-O')]
             self.assertEqual(Oargs, [Oflag, '-O0'])
 
-    def test_custom_target_exe_data_deterministic(self):
-        testdir = os.path.join(self.common_test_dir, '117 custom target capture')
-        self.init(testdir)
-        meson_exe_dat1 = glob(os.path.join(self.privatedir, 'meson_exe*.dat'))
-        self.wipe()
-        self.init(testdir)
-        meson_exe_dat2 = glob(os.path.join(self.privatedir, 'meson_exe*.dat'))
-        self.assertListEqual(meson_exe_dat1, meson_exe_dat2)
-
     def _test_stds_impl(self, testdir, compiler, p):
         lang_std = p + '_std'
         # Check that all the listed -std=xxx options for this compiler work
@@ -1154,6 +1388,14 @@ class LinuxlikeTests(BasePlatformTests):
         cpp = env.detect_cpp_compiler(False)
         self._test_stds_impl(testdir, cpp, 'cpp')
 
+    def test_unity_subproj(self):
+        testdir = os.path.join(self.common_test_dir, '49 subproject')
+        self.init(testdir, extra_args='--unity=subprojects')
+        self.assertTrue(os.path.exists(os.path.join(self.builddir, 'subprojects/sublib/simpletest@exe/simpletest-unity.c')))
+        self.assertTrue(os.path.exists(os.path.join(self.builddir, 'subprojects/sublib/sublib@sha/sublib-unity.c')))
+        self.assertFalse(os.path.exists(os.path.join(self.builddir, 'user@exe/user-unity.c')))
+        self.build()
+
     def test_installed_modes(self):
         '''
         Test that files installed by these tests have the correct permissions.
@@ -1207,6 +1449,49 @@ class LinuxlikeTests(BasePlatformTests):
             # The chown failed nonfatally if we're not root
             self.assertEqual(0, statf.st_uid)
 
+    def test_cpp_std_override(self):
+        testdir = os.path.join(self.unit_test_dir, '6 std override')
+        self.init(testdir)
+        compdb = self.get_compdb()
+        for i in compdb:
+            if 'prog03' in i['file']:
+                c03_comp = i['command']
+            if 'prog11' in i['file']:
+                c11_comp = i['command']
+            if 'progp' in i['file']:
+                plain_comp = i['command']
+        self.assertNotEqual(len(plain_comp), 0)
+        self.assertIn('-std=c++03', c03_comp)
+        self.assertNotIn('-std=c++11', c03_comp)
+        self.assertIn('-std=c++11', c11_comp)
+        self.assertNotIn('-std=c++03', c11_comp)
+        self.assertNotIn('-std=c++03', plain_comp)
+        self.assertNotIn('-std=c++11', plain_comp)
+        # Now werror
+        self.assertIn('-Werror', plain_comp)
+        self.assertNotIn('-Werror', c03_comp)
+
+    def test_run_installed(self):
+        testdir = os.path.join(self.unit_test_dir, '7 run installed')
+        self.init(testdir)
+        self.build()
+        self.install()
+        installed_exe = os.path.join(self.installdir, 'usr/bin/prog')
+        installed_libdir = os.path.join(self.installdir, 'usr/foo')
+        installed_lib = os.path.join(installed_libdir, 'libfoo.so')
+        self.assertTrue(os.path.isfile(installed_exe))
+        self.assertTrue(os.path.isdir(installed_libdir))
+        self.assertTrue(os.path.isfile(installed_lib))
+        # Must fail when run without LD_LIBRARY_PATH to ensure that
+        # rpath has been properly stripped rather than pointing to the builddir.
+        self.assertNotEqual(subprocess.call(installed_exe, stderr=subprocess.DEVNULL), 0)
+        # When LD_LIBRARY_PATH is set it should start working.
+        # For some reason setting LD_LIBRARY_PATH in os.environ fails
+        # when all tests are run (but works when only this test is run),
+        # but doing this explicitly works.
+        env = os.environ.copy()
+        env['LD_LIBRARY_PATH'] = installed_libdir
+        self.assertEqual(subprocess.call(installed_exe, env=env), 0)
 
 class RewriterTests(unittest.TestCase):
 
@@ -1271,5 +1556,15 @@ class RewriterTests(unittest.TestCase):
         self.assertEqual(s2, self.read_contents('sub2/meson.build'))
 
 
+def unset_envs():
+    # For unit tests we must fully control all commend lines
+    # so that there are no unexpected changes coming from the
+    # environment, for example when doing a package build.
+    varnames = ['CPPFLAGS', 'LDFLAGS'] + list(mesonbuild.environment.cflags_mapping.values())
+    for v in varnames:
+        if v in os.environ:
+            del os.environ[v]
+
 if __name__ == '__main__':
+    unset_envs()
     unittest.main(buffer=True)
