@@ -1,6 +1,6 @@
-#!/usr/bin/env python3
+#!/usr/bin/env python
 
-import sys, os
+import sys
 
 with open(sys.argv[1], 'rb') as ifile:
     with open(sys.argv[2], 'w') as ofile:
