@@ -24,7 +24,7 @@ class Token:
         self.lineno = 0
         self.colno = 0
 
-class Statement():
+class Statement:
     def __init__(self, name, args):
         self.name = name
         self.args = args
@@ -38,7 +38,7 @@ class Lexer:
             ('varexp', re.compile(r'\${[-_0-9a-z/A-Z.]+}')),
             ('id', re.compile('''[,-><${}=+_0-9a-z/A-Z|@.*]+''')),
             ('eol', re.compile(r'\n')),
-            ('comment', re.compile(r'\#.*')),
+            ('comment', re.compile(r'#.*')),
             ('lparen', re.compile(r'\(')),
             ('rparen', re.compile(r'\)')),
         ]
@@ -46,14 +46,14 @@ class Lexer:
     def lex(self, code):
         lineno = 1
         line_start = 0
-        loc = 0;
+        loc = 0
         col = 0
-        while(loc < len(code)):
+        while loc < len(code):
             matched = False
             for (tid, reg) in self.token_specification:
                 mo = reg.match(code, loc)
                 if mo:
-                    col = mo.start()-line_start
+                    col = mo.start() - line_start
                     matched = True
                     loc = mo.end()
                     match_text = mo.group()
@@ -70,7 +70,7 @@ class Lexer:
                     elif tid == 'id':
                         yield(Token('id', match_text))
                     elif tid == 'eol':
-                        #yield('eol')
+                        # yield('eol')
                         lineno += 1
                         col = 1
                         line_start = mo.end()
@@ -83,7 +83,7 @@ class Lexer:
             if not matched:
                 raise RuntimeError('Lexer got confused line %d column %d' % (lineno, col))
 
-class Parser():
+class Parser:
     def __init__(self, code):
         self.stream = Lexer().lex(code)
         self.getsym()
@@ -121,8 +121,9 @@ class Parser():
             args.append(self.arguments())
             self.expect('rparen')
         arg = self.current
-        if self.accept('string') or self.accept('varexp') or\
-        self.accept('id'):
+        if self.accept('string') \
+                or self.accept('varexp') \
+                or self.accept('id'):
             args.append(arg)
             rest = self.arguments()
             args += rest
@@ -133,9 +134,10 @@ class Parser():
             yield(self.statement())
 
 class Converter:
-    ignored_funcs = {'cmake_minimum_required' : True,
-                     'enable_testing' : True,
-                     'include' : True}
+    ignored_funcs = {'cmake_minimum_required': True,
+                     'enable_testing': True,
+                     'include': True}
+
     def __init__(self, cmake_root):
         self.cmake_root = cmake_root
         self.indent_unit = '  '
@@ -240,7 +242,7 @@ class Converter:
         else:
             line = '''# %s(%s)''' % (t.name, self.convert_args(t.args))
         self.indent_level += preincrement
-        indent = self.indent_level*self.indent_unit
+        indent = self.indent_level * self.indent_unit
         outfile.write(indent)
         outfile.write(line)
         if not(line.endswith('\n')):
@@ -276,6 +278,7 @@ class Converter:
             for o in self.options:
                 (optname, description, default) = o
                 if default is None:
+                    typestr = ''
                     defaultstr = ''
                 else:
                     if default == 'OFF':
